/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "wpa_supplicant-interface.h"

#include <string.h>
#ifdef G_OS_UNIX
#  include <gio/gunixfdlist.h>
#endif

#ifdef G_ENABLE_DEBUG
#define g_marshal_value_peek_boolean(v)  g_value_get_boolean (v)
#define g_marshal_value_peek_char(v)     g_value_get_schar (v)
#define g_marshal_value_peek_uchar(v)    g_value_get_uchar (v)
#define g_marshal_value_peek_int(v)      g_value_get_int (v)
#define g_marshal_value_peek_uint(v)     g_value_get_uint (v)
#define g_marshal_value_peek_long(v)     g_value_get_long (v)
#define g_marshal_value_peek_ulong(v)    g_value_get_ulong (v)
#define g_marshal_value_peek_int64(v)    g_value_get_int64 (v)
#define g_marshal_value_peek_uint64(v)   g_value_get_uint64 (v)
#define g_marshal_value_peek_enum(v)     g_value_get_enum (v)
#define g_marshal_value_peek_flags(v)    g_value_get_flags (v)
#define g_marshal_value_peek_float(v)    g_value_get_float (v)
#define g_marshal_value_peek_double(v)   g_value_get_double (v)
#define g_marshal_value_peek_string(v)   (char*) g_value_get_string (v)
#define g_marshal_value_peek_param(v)    g_value_get_param (v)
#define g_marshal_value_peek_boxed(v)    g_value_get_boxed (v)
#define g_marshal_value_peek_pointer(v)  g_value_get_pointer (v)
#define g_marshal_value_peek_object(v)   g_value_get_object (v)
#define g_marshal_value_peek_variant(v)  g_value_get_variant (v)
#else /* !G_ENABLE_DEBUG */
/* WARNING: This code accesses GValues directly, which is UNSUPPORTED API.
 *          Do not access GValues directly in your code. Instead, use the
 *          g_value_get_*() functions
 */
#define g_marshal_value_peek_boolean(v)  (v)->data[0].v_int
#define g_marshal_value_peek_char(v)     (v)->data[0].v_int
#define g_marshal_value_peek_uchar(v)    (v)->data[0].v_uint
#define g_marshal_value_peek_int(v)      (v)->data[0].v_int
#define g_marshal_value_peek_uint(v)     (v)->data[0].v_uint
#define g_marshal_value_peek_long(v)     (v)->data[0].v_long
#define g_marshal_value_peek_ulong(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_int64(v)    (v)->data[0].v_int64
#define g_marshal_value_peek_uint64(v)   (v)->data[0].v_uint64
#define g_marshal_value_peek_enum(v)     (v)->data[0].v_long
#define g_marshal_value_peek_flags(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_float(v)    (v)->data[0].v_float
#define g_marshal_value_peek_double(v)   (v)->data[0].v_double
#define g_marshal_value_peek_string(v)   (v)->data[0].v_pointer
#define g_marshal_value_peek_param(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_boxed(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_pointer(v)  (v)->data[0].v_pointer
#define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer
#define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
#endif /* !G_ENABLE_DEBUG */

typedef struct
{
  GDBusArgInfo parent_struct;
  gboolean use_gvariant;
} _ExtendedGDBusArgInfo;

typedef struct
{
  GDBusMethodInfo parent_struct;
  const gchar *signal_name;
  gboolean pass_fdlist;
} _ExtendedGDBusMethodInfo;

typedef struct
{
  GDBusSignalInfo parent_struct;
  const gchar *signal_name;
} _ExtendedGDBusSignalInfo;

typedef struct
{
  GDBusPropertyInfo parent_struct;
  const gchar *hyphen_name;
  guint use_gvariant : 1;
  guint emits_changed_signal : 1;
} _ExtendedGDBusPropertyInfo;

typedef struct
{
  GDBusInterfaceInfo parent_struct;
  const gchar *hyphen_name;
} _ExtendedGDBusInterfaceInfo;

typedef struct
{
  const _ExtendedGDBusPropertyInfo *info;
  guint prop_id;
  GValue orig_value; /* the value before the change */
} ChangedProperty;

static void
_changed_property_free (ChangedProperty *data)
{
  g_value_unset (&data->orig_value);
  g_free (data);
}

static gboolean
_g_strv_equal0 (gchar **a, gchar **b)
{
  gboolean ret = FALSE;
  guint n;
  if (a == NULL && b == NULL)
    {
      ret = TRUE;
      goto out;
    }
  if (a == NULL || b == NULL)
    goto out;
  if (g_strv_length (a) != g_strv_length (b))
    goto out;
  for (n = 0; a[n] != NULL; n++)
    if (g_strcmp0 (a[n], b[n]) != 0)
      goto out;
  ret = TRUE;
out:
  return ret;
}

static gboolean
_g_variant_equal0 (GVariant *a, GVariant *b)
{
  gboolean ret = FALSE;
  if (a == NULL && b == NULL)
    {
      ret = TRUE;
      goto out;
    }
  if (a == NULL || b == NULL)
    goto out;
  ret = g_variant_equal (a, b);
out:
  return ret;
}

G_GNUC_UNUSED static gboolean
_g_value_equal (const GValue *a, const GValue *b)
{
  gboolean ret = FALSE;
  g_assert (G_VALUE_TYPE (a) == G_VALUE_TYPE (b));
  switch (G_VALUE_TYPE (a))
    {
      case G_TYPE_BOOLEAN:
        ret = (g_value_get_boolean (a) == g_value_get_boolean (b));
        break;
      case G_TYPE_UCHAR:
        ret = (g_value_get_uchar (a) == g_value_get_uchar (b));
        break;
      case G_TYPE_INT:
        ret = (g_value_get_int (a) == g_value_get_int (b));
        break;
      case G_TYPE_UINT:
        ret = (g_value_get_uint (a) == g_value_get_uint (b));
        break;
      case G_TYPE_INT64:
        ret = (g_value_get_int64 (a) == g_value_get_int64 (b));
        break;
      case G_TYPE_UINT64:
        ret = (g_value_get_uint64 (a) == g_value_get_uint64 (b));
        break;
      case G_TYPE_DOUBLE:
        {
          /* Avoid -Wfloat-equal warnings by doing a direct bit compare */
          gdouble da = g_value_get_double (a);
          gdouble db = g_value_get_double (b);
          ret = memcmp (&da, &db, sizeof (gdouble)) == 0;
        }
        break;
      case G_TYPE_STRING:
        ret = (g_strcmp0 (g_value_get_string (a), g_value_get_string (b)) == 0);
        break;
      case G_TYPE_VARIANT:
        ret = _g_variant_equal0 (g_value_get_variant (a), g_value_get_variant (b));
        break;
      default:
        if (G_VALUE_TYPE (a) == G_TYPE_STRV)
          ret = _g_strv_equal0 (g_value_get_boxed (a), g_value_get_boxed (b));
        else
          g_critical ("_g_value_equal() does not handle type %s", g_type_name (G_VALUE_TYPE (a)));
        break;
    }
  return ret;
}

static void
_g_dbus_codegen_marshal_VOID__STRING_VARIANT (
    GClosure     *closure,
    GValue       *return_value G_GNUC_UNUSED,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef void (*_GDbusCodegenMarshalVoid_StringVariantFunc)
       (void *data1,
        const gchar *arg_path,
        GVariant *arg_properties,
        void *data2);
  _GDbusCodegenMarshalVoid_StringVariantFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;

  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalVoid_StringVariantFunc)
    (marshal_data ? marshal_data : cc->callback);

  callback (data1,
            g_marshal_value_peek_string (param_values + 1),
            g_marshal_value_peek_variant (param_values + 2),
            data2);
}

static void
_g_dbus_codegen_marshal_BOOLEAN__OBJECT_VARIANT (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint G_GNUC_UNUSED,
    void         *marshal_data)
{
  typedef gboolean (*_GDbusCodegenMarshalBoolean_ObjectVariantFunc)
       (void *data1,
        GDBusMethodInvocation *arg_method_invocation,
        GVariant *arg_args,
        void *data2);
  _GDbusCodegenMarshalBoolean_ObjectVariantFunc callback;
  GCClosure *cc = (GCClosure*) closure;
  void *data1, *data2;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }

  callback = (_GDbusCodegenMarshalBoolean_ObjectVariantFunc)
    (marshal_data ? marshal_data : cc->callback);

  v_return =
    callback (data1,
              g_marshal_value_peek_object (param_values + 1),
              g_marshal_value_peek_variant (param_values + 2),
              data2);

  g_value_set_boolean (return_value, v_return);
}

/* ------------------------------------------------------------------------
 * Code for interface fi.w1.wpa_supplicant1
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:WPASupplicant
 * @title: WPASupplicant
 * @short_description: Generated C code for the fi.w1.wpa_supplicant1 D-Bus interface
 *
 * This section contains code for working with the <link linkend="gdbus-interface-fi-w1-wpa_supplicant1.top_of_page">fi.w1.wpa_supplicant1</link> D-Bus interface in C.
 */

enum
{
  _WPA_SUPPLICANT_INTERFACE_ADDED,
  _WPA_SUPPLICANT_INTERFACE_REMOVED,
};

static unsigned _WPA_SUPPLICANT_SIGNALS[2] = { 0 };

/* ---- Introspection data for fi.w1.wpa_supplicant1 ---- */

static const _ExtendedGDBusArgInfo _wpa_supplicant_signal_info_interface_added_ARG_path =
{
  {
    -1,
    (gchar *) "path",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _wpa_supplicant_signal_info_interface_added_ARG_properties =
{
  {
    -1,
    (gchar *) "properties",
    (gchar *) "a{sv}",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _wpa_supplicant_signal_info_interface_added_ARG_pointers[] =
{
  &_wpa_supplicant_signal_info_interface_added_ARG_path.parent_struct,
  &_wpa_supplicant_signal_info_interface_added_ARG_properties.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _wpa_supplicant_signal_info_interface_added =
{
  {
    -1,
    (gchar *) "InterfaceAdded",
    (GDBusArgInfo **) &_wpa_supplicant_signal_info_interface_added_ARG_pointers,
    NULL
  },
  "interface-added"
};

static const _ExtendedGDBusArgInfo _wpa_supplicant_signal_info_interface_removed_ARG_path =
{
  {
    -1,
    (gchar *) "path",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _wpa_supplicant_signal_info_interface_removed_ARG_pointers[] =
{
  &_wpa_supplicant_signal_info_interface_removed_ARG_path.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _wpa_supplicant_signal_info_interface_removed =
{
  {
    -1,
    (gchar *) "InterfaceRemoved",
    (GDBusArgInfo **) &_wpa_supplicant_signal_info_interface_removed_ARG_pointers,
    NULL
  },
  "interface-removed"
};

static const GDBusSignalInfo * const _wpa_supplicant_signal_info_pointers[] =
{
  &_wpa_supplicant_signal_info_interface_added.parent_struct,
  &_wpa_supplicant_signal_info_interface_removed.parent_struct,
  NULL
};

static const _ExtendedGDBusPropertyInfo _wpa_supplicant_property_info_interfaces =
{
  {
    -1,
    (gchar *) "Interfaces",
    (gchar *) "ao",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "interfaces",
  FALSE,
  TRUE
};

static const GDBusPropertyInfo * const _wpa_supplicant_property_info_pointers[] =
{
  &_wpa_supplicant_property_info_interfaces.parent_struct,
  NULL
};

static const _ExtendedGDBusInterfaceInfo _wpa_supplicant_interface_info =
{
  {
    -1,
    (gchar *) "fi.w1.wpa_supplicant1",
    NULL,
    (GDBusSignalInfo **) &_wpa_supplicant_signal_info_pointers,
    (GDBusPropertyInfo **) &_wpa_supplicant_property_info_pointers,
    NULL
  },
  "wpa-supplicant",
};


/**
 * wpa_supplicant_interface_info:
 *
 * Gets a machine-readable description of the <link linkend="gdbus-interface-fi-w1-wpa_supplicant1.top_of_page">fi.w1.wpa_supplicant1</link> D-Bus interface.
 *
 * Returns: (transfer none): A #GDBusInterfaceInfo. Do not free.
 */
GDBusInterfaceInfo *
wpa_supplicant_interface_info (void)
{
  return (GDBusInterfaceInfo *) &_wpa_supplicant_interface_info.parent_struct;
}

/**
 * wpa_supplicant_override_properties:
 * @klass: The class structure for a #GObject derived class.
 * @property_id_begin: The property id to assign to the first overridden property.
 *
 * Overrides all #GObject properties in the #WPASupplicant interface for a concrete class.
 * The properties are overridden in the order they are defined.
 *
 * Returns: The last property id.
 */
guint
wpa_supplicant_override_properties (GObjectClass *klass, guint property_id_begin)
{
  g_object_class_override_property (klass, property_id_begin++, "interfaces");
  return property_id_begin - 1;
}


inline static void
wpa_supplicant_signal_marshal_interface_added (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_VOID__STRING_VARIANT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
wpa_supplicant_signal_marshal_interface_removed (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}


/**
 * WPASupplicant:
 *
 * Abstract interface type for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1.top_of_page">fi.w1.wpa_supplicant1</link>.
 */

/**
 * WPASupplicantIface:
 * @parent_iface: The parent interface.
 * @get_interfaces: Getter for the #WPASupplicant:interfaces property.
 * @interface_added: Handler for the #WPASupplicant::interface-added signal.
 * @interface_removed: Handler for the #WPASupplicant::interface-removed signal.
 *
 * Virtual table for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1.top_of_page">fi.w1.wpa_supplicant1</link>.
 */

typedef WPASupplicantIface WPASupplicantInterface;
G_DEFINE_INTERFACE (WPASupplicant, wpa_supplicant, G_TYPE_OBJECT)

static void
wpa_supplicant_default_init (WPASupplicantIface *iface)
{
  /* GObject signals for received D-Bus signals: */
  /**
   * WPASupplicant::interface-added:
   * @object: A #WPASupplicant.
   * @arg_path: Argument.
   * @arg_properties: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-fi-w1-wpa_supplicant1.InterfaceAdded">"InterfaceAdded"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _WPA_SUPPLICANT_SIGNALS[_WPA_SUPPLICANT_INTERFACE_ADDED] =
    g_signal_new ("interface-added",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (WPASupplicantIface, interface_added),
      NULL,
      NULL,
      wpa_supplicant_signal_marshal_interface_added,
      G_TYPE_NONE,
      2, G_TYPE_STRING, G_TYPE_VARIANT);

  /**
   * WPASupplicant::interface-removed:
   * @object: A #WPASupplicant.
   * @arg_path: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-fi-w1-wpa_supplicant1.InterfaceRemoved">"InterfaceRemoved"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _WPA_SUPPLICANT_SIGNALS[_WPA_SUPPLICANT_INTERFACE_REMOVED] =
    g_signal_new ("interface-removed",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (WPASupplicantIface, interface_removed),
      NULL,
      NULL,
      wpa_supplicant_signal_marshal_interface_removed,
      G_TYPE_NONE,
      1, G_TYPE_STRING);

  /* GObject properties for D-Bus properties: */
  /**
   * WPASupplicant:interfaces:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-fi-w1-wpa_supplicant1.Interfaces">"Interfaces"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_boxed ("interfaces", "Interfaces", "Interfaces", G_TYPE_STRV, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
}

/**
 * wpa_supplicant_get_interfaces: (skip)
 * @object: A #WPASupplicant.
 *
 * Gets the value of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1.Interfaces">"Interfaces"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use wpa_supplicant_dup_interfaces() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *const *
wpa_supplicant_get_interfaces (WPASupplicant *object)
{
  g_return_val_if_fail (IS_WPA_SUPPLICANT (object), NULL);

  return WPA_SUPPLICANT_GET_IFACE (object)->get_interfaces (object);
}

/**
 * wpa_supplicant_dup_interfaces: (skip)
 * @object: A #WPASupplicant.
 *
 * Gets a copy of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1.Interfaces">"Interfaces"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_strfreev().
 */
gchar **
wpa_supplicant_dup_interfaces (WPASupplicant *object)
{
  gchar **value;
  g_object_get (G_OBJECT (object), "interfaces", &value, NULL);
  return value;
}

/**
 * wpa_supplicant_set_interfaces: (skip)
 * @object: A #WPASupplicant.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-fi-w1-wpa_supplicant1.Interfaces">"Interfaces"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
wpa_supplicant_set_interfaces (WPASupplicant *object, const gchar *const *value)
{
  g_object_set (G_OBJECT (object), "interfaces", value, NULL);
}

/**
 * wpa_supplicant_emit_interface_added:
 * @object: A #WPASupplicant.
 * @arg_path: Argument to pass with the signal.
 * @arg_properties: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-fi-w1-wpa_supplicant1.InterfaceAdded">"InterfaceAdded"</link> D-Bus signal.
 */
void
wpa_supplicant_emit_interface_added (
    WPASupplicant *object,
    const gchar *arg_path,
    GVariant *arg_properties)
{
  g_signal_emit (object, _WPA_SUPPLICANT_SIGNALS[_WPA_SUPPLICANT_INTERFACE_ADDED], 0, arg_path, arg_properties);
}

/**
 * wpa_supplicant_emit_interface_removed:
 * @object: A #WPASupplicant.
 * @arg_path: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-fi-w1-wpa_supplicant1.InterfaceRemoved">"InterfaceRemoved"</link> D-Bus signal.
 */
void
wpa_supplicant_emit_interface_removed (
    WPASupplicant *object,
    const gchar *arg_path)
{
  g_signal_emit (object, _WPA_SUPPLICANT_SIGNALS[_WPA_SUPPLICANT_INTERFACE_REMOVED], 0, arg_path);
}

/* ------------------------------------------------------------------------ */

/**
 * WPASupplicantProxy:
 *
 * The #WPASupplicantProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * WPASupplicantProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #WPASupplicantProxy.
 */

struct _WPASupplicantProxyPrivate
{
  GData *qdata;
};

static void wpa_supplicant_proxy_iface_init (WPASupplicantIface *iface);

#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (WPASupplicantProxy, wpa_supplicant_proxy, G_TYPE_DBUS_PROXY,
                         G_ADD_PRIVATE (WPASupplicantProxy)
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_SUPPLICANT, wpa_supplicant_proxy_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (WPASupplicantProxy, wpa_supplicant_proxy, G_TYPE_DBUS_PROXY,
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_SUPPLICANT, wpa_supplicant_proxy_iface_init))

#endif
static void
wpa_supplicant_proxy_finalize (GObject *object)
{
  WPASupplicantProxy *proxy = WPA_SUPPLICANT_PROXY (object);
  g_datalist_clear (&proxy->priv->qdata);
  G_OBJECT_CLASS (wpa_supplicant_proxy_parent_class)->finalize (object);
}

static void
wpa_supplicant_proxy_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 1);
  info = (const _ExtendedGDBusPropertyInfo *) _wpa_supplicant_property_info_pointers[prop_id - 1];
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (object), info->parent_struct.name);
  if (info->use_gvariant)
    {
      g_value_set_variant (value, variant);
    }
  else
    {
      if (variant != NULL)
        g_dbus_gvariant_to_gvalue (variant, value);
    }
  if (variant != NULL)
    g_variant_unref (variant);
}

static void
wpa_supplicant_proxy_set_property_cb (GDBusProxy *proxy,
  GAsyncResult *res,
  gpointer      user_data)
{
  const _ExtendedGDBusPropertyInfo *info = user_data;
  GError *error;
  GVariant *_ret;
  error = NULL;
  _ret = g_dbus_proxy_call_finish (proxy, res, &error);
  if (!_ret)
    {
      g_warning ("Error setting property '%s' on interface fi.w1.wpa_supplicant1: %s (%s, %d)",
                 info->parent_struct.name, 
                 error->message, g_quark_to_string (error->domain), error->code);
      g_error_free (error);
    }
  else
    {
      g_variant_unref (_ret);
    }
}

static void
wpa_supplicant_proxy_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 1);
  info = (const _ExtendedGDBusPropertyInfo *) _wpa_supplicant_property_info_pointers[prop_id - 1];
  variant = g_dbus_gvalue_to_gvariant (value, G_VARIANT_TYPE (info->parent_struct.signature));
  g_dbus_proxy_call (G_DBUS_PROXY (object),
    "org.freedesktop.DBus.Properties.Set",
    g_variant_new ("(ssv)", "fi.w1.wpa_supplicant1", info->parent_struct.name, variant),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    NULL, (GAsyncReadyCallback) wpa_supplicant_proxy_set_property_cb, (GDBusPropertyInfo *) &info->parent_struct);
  g_variant_unref (variant);
}

static void
wpa_supplicant_proxy_g_signal (GDBusProxy *proxy,
  const gchar *sender_name G_GNUC_UNUSED,
  const gchar *signal_name,
  GVariant *parameters)
{
  _ExtendedGDBusSignalInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  gsize n;
  guint signal_id;
  info = (_ExtendedGDBusSignalInfo *) g_dbus_interface_info_lookup_signal ((GDBusInterfaceInfo *) &_wpa_supplicant_interface_info.parent_struct, signal_name);
  if (info == NULL)
    return;
  num_params = g_variant_n_children (parameters);
  paramv = g_new0 (GValue, num_params + 1);
  g_value_init (&paramv[0], TYPE_WPA_SUPPLICANT);
  g_value_set_object (&paramv[0], proxy);
  g_variant_iter_init (&iter, parameters);
  n = 1;
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.args[n - 1];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_WPA_SUPPLICANT);
  g_signal_emitv (paramv, signal_id, 0, NULL);
  for (n = 0; n < num_params + 1; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static void
wpa_supplicant_proxy_g_properties_changed (GDBusProxy *_proxy,
  GVariant *changed_properties,
  const gchar *const *invalidated_properties)
{
  WPASupplicantProxy *proxy = WPA_SUPPLICANT_PROXY (_proxy);
  guint n;
  const gchar *key;
  GVariantIter *iter;
  _ExtendedGDBusPropertyInfo *info;
  g_variant_get (changed_properties, "a{sv}", &iter);
  while (g_variant_iter_next (iter, "{&sv}", &key, NULL))
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_supplicant_interface_info.parent_struct, key);
      g_datalist_remove_data (&proxy->priv->qdata, key);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
  g_variant_iter_free (iter);
  for (n = 0; invalidated_properties[n] != NULL; n++)
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_supplicant_interface_info.parent_struct, invalidated_properties[n]);
      g_datalist_remove_data (&proxy->priv->qdata, invalidated_properties[n]);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
}

static const gchar *const *
wpa_supplicant_proxy_get_interfaces (WPASupplicant *object)
{
  WPASupplicantProxy *proxy = WPA_SUPPLICANT_PROXY (object);
  GVariant *variant;
  const gchar *const *value = NULL;
  value = g_datalist_get_data (&proxy->priv->qdata, "Interfaces");
  if (value != NULL)
    return value;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Interfaces");
  if (variant != NULL)
    {
      value = g_variant_get_objv (variant, NULL);
      g_datalist_set_data_full (&proxy->priv->qdata, "Interfaces", (gpointer) value, g_free);
      g_variant_unref (variant);
    }
  return value;
}

static void
wpa_supplicant_proxy_init (WPASupplicantProxy *proxy)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  proxy->priv = wpa_supplicant_proxy_get_instance_private (proxy);
#else
  proxy->priv = G_TYPE_INSTANCE_GET_PRIVATE (proxy, TYPE_WPA_SUPPLICANT_PROXY, WPASupplicantProxyPrivate);
#endif

  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (proxy), wpa_supplicant_interface_info ());
}

static void
wpa_supplicant_proxy_class_init (WPASupplicantProxyClass *klass)
{
  GObjectClass *gobject_class;
  GDBusProxyClass *proxy_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize     = wpa_supplicant_proxy_finalize;
  gobject_class->get_property = wpa_supplicant_proxy_get_property;
  gobject_class->set_property = wpa_supplicant_proxy_set_property;

  proxy_class = G_DBUS_PROXY_CLASS (klass);
  proxy_class->g_signal = wpa_supplicant_proxy_g_signal;
  proxy_class->g_properties_changed = wpa_supplicant_proxy_g_properties_changed;

  wpa_supplicant_override_properties (gobject_class, 1);

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (WPASupplicantProxyPrivate));
#endif
}

static void
wpa_supplicant_proxy_iface_init (WPASupplicantIface *iface)
{
  iface->get_interfaces = wpa_supplicant_proxy_get_interfaces;
}

/**
 * wpa_supplicant_proxy_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1.top_of_page">fi.w1.wpa_supplicant1</link>. See g_dbus_proxy_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call wpa_supplicant_proxy_new_finish() to get the result of the operation.
 *
 * See wpa_supplicant_proxy_new_sync() for the synchronous, blocking version of this constructor.
 */
void
wpa_supplicant_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_WPA_SUPPLICANT_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1", NULL);
}

/**
 * wpa_supplicant_proxy_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to wpa_supplicant_proxy_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with wpa_supplicant_proxy_new().
 *
 * Returns: (transfer full) (type WPASupplicantProxy): The constructed proxy object or %NULL if @error is set.
 */
WPASupplicant *
wpa_supplicant_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return WPA_SUPPLICANT (ret);
  else
    return NULL;
}

/**
 * wpa_supplicant_proxy_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1.top_of_page">fi.w1.wpa_supplicant1</link>. See g_dbus_proxy_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See wpa_supplicant_proxy_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type WPASupplicantProxy): The constructed proxy object or %NULL if @error is set.
 */
WPASupplicant *
wpa_supplicant_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_WPA_SUPPLICANT_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1", NULL);
  if (ret != NULL)
    return WPA_SUPPLICANT (ret);
  else
    return NULL;
}


/**
 * wpa_supplicant_proxy_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like wpa_supplicant_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call wpa_supplicant_proxy_new_for_bus_finish() to get the result of the operation.
 *
 * See wpa_supplicant_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
wpa_supplicant_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_WPA_SUPPLICANT_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1", NULL);
}

/**
 * wpa_supplicant_proxy_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to wpa_supplicant_proxy_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with wpa_supplicant_proxy_new_for_bus().
 *
 * Returns: (transfer full) (type WPASupplicantProxy): The constructed proxy object or %NULL if @error is set.
 */
WPASupplicant *
wpa_supplicant_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return WPA_SUPPLICANT (ret);
  else
    return NULL;
}

/**
 * wpa_supplicant_proxy_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like wpa_supplicant_proxy_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See wpa_supplicant_proxy_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type WPASupplicantProxy): The constructed proxy object or %NULL if @error is set.
 */
WPASupplicant *
wpa_supplicant_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_WPA_SUPPLICANT_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1", NULL);
  if (ret != NULL)
    return WPA_SUPPLICANT (ret);
  else
    return NULL;
}


/* ------------------------------------------------------------------------ */

/**
 * WPASupplicantSkeleton:
 *
 * The #WPASupplicantSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * WPASupplicantSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #WPASupplicantSkeleton.
 */

struct _WPASupplicantSkeletonPrivate
{
  GValue *properties;
  GList *changed_properties;
  GSource *changed_properties_idle_source;
  GMainContext *context;
  GMutex lock;
};

static void
_wpa_supplicant_skeleton_handle_method_call (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name,
  const gchar *method_name,
  GVariant *parameters,
  GDBusMethodInvocation *invocation,
  gpointer user_data)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (user_data);
  _ExtendedGDBusMethodInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  guint num_extra;
  gsize n;
  guint signal_id;
  GValue return_value = G_VALUE_INIT;
  info = (_ExtendedGDBusMethodInfo *) g_dbus_method_invocation_get_method_info (invocation);
  g_assert (info != NULL);
  num_params = g_variant_n_children (parameters);
  num_extra = info->pass_fdlist ? 3 : 2;  paramv = g_new0 (GValue, num_params + num_extra);
  n = 0;
  g_value_init (&paramv[n], TYPE_WPA_SUPPLICANT);
  g_value_set_object (&paramv[n++], skeleton);
  g_value_init (&paramv[n], G_TYPE_DBUS_METHOD_INVOCATION);
  g_value_set_object (&paramv[n++], invocation);
  if (info->pass_fdlist)
    {
#ifdef G_OS_UNIX
      g_value_init (&paramv[n], G_TYPE_UNIX_FD_LIST);
      g_value_set_object (&paramv[n++], g_dbus_message_get_unix_fd_list (g_dbus_method_invocation_get_message (invocation)));
#else
      g_assert_not_reached ();
#endif
    }
  g_variant_iter_init (&iter, parameters);
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.in_args[n - num_extra];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_WPA_SUPPLICANT);
  g_value_init (&return_value, G_TYPE_BOOLEAN);
  g_signal_emitv (paramv, signal_id, 0, &return_value);
  if (!g_value_get_boolean (&return_value))
    g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Method %s is not implemented on interface %s", method_name, interface_name);
  g_value_unset (&return_value);
  for (n = 0; n < num_params + num_extra; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static GVariant *
_wpa_supplicant_skeleton_handle_get_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GError **error,
  gpointer user_data)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  GVariant *ret;
  ret = NULL;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_supplicant_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      g_value_init (&value, pspec->value_type);
      g_object_get_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      ret = g_dbus_gvalue_to_gvariant (&value, G_VARIANT_TYPE (info->parent_struct.signature));
      g_value_unset (&value);
    }
  return ret;
}

static gboolean
_wpa_supplicant_skeleton_handle_set_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GVariant *variant,
  GError **error,
  gpointer user_data)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  gboolean ret;
  ret = FALSE;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_supplicant_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      if (info->use_gvariant)
        g_value_set_variant (&value, variant);
      else
        g_dbus_gvariant_to_gvalue (variant, &value);
      g_object_set_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      g_value_unset (&value);
      ret = TRUE;
    }
  return ret;
}

static const GDBusInterfaceVTable _wpa_supplicant_skeleton_vtable =
{
  _wpa_supplicant_skeleton_handle_method_call,
  _wpa_supplicant_skeleton_handle_get_property,
  _wpa_supplicant_skeleton_handle_set_property,
  {NULL}
};

static GDBusInterfaceInfo *
wpa_supplicant_skeleton_dbus_interface_get_info (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return wpa_supplicant_interface_info ();
}

static GDBusInterfaceVTable *
wpa_supplicant_skeleton_dbus_interface_get_vtable (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return (GDBusInterfaceVTable *) &_wpa_supplicant_skeleton_vtable;
}

static GVariant *
wpa_supplicant_skeleton_dbus_interface_get_properties (GDBusInterfaceSkeleton *_skeleton)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (_skeleton);

  GVariantBuilder builder;
  guint n;
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
#else
  g_variant_builder_init(&builder, G_VARIANT_TYPE ("a{sv}"));
#endif
  if (_wpa_supplicant_interface_info.parent_struct.properties == NULL)
    goto out;
  for (n = 0; _wpa_supplicant_interface_info.parent_struct.properties[n] != NULL; n++)
    {
      GDBusPropertyInfo *info = _wpa_supplicant_interface_info.parent_struct.properties[n];
      if (info->flags & G_DBUS_PROPERTY_INFO_FLAGS_READABLE)
        {
          GVariant *value;
          value = _wpa_supplicant_skeleton_handle_get_property (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)), NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "fi.w1.wpa_supplicant1", info->name, NULL, skeleton);
          if (value != NULL)
            {
              g_variant_take_ref (value);
              g_variant_builder_add (&builder, "{sv}", info->name, value);
              g_variant_unref (value);
            }
        }
    }
out:
  return g_variant_builder_end (&builder);
}

static gboolean _wpa_supplicant_emit_changed (gpointer user_data);

static void
wpa_supplicant_skeleton_dbus_interface_flush (GDBusInterfaceSkeleton *_skeleton)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (_skeleton);
  gboolean emit_changed = FALSE;

  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    {
      g_source_destroy (skeleton->priv->changed_properties_idle_source);
      skeleton->priv->changed_properties_idle_source = NULL;
      emit_changed = TRUE;
    }
  g_mutex_unlock (&skeleton->priv->lock);

  if (emit_changed)
    _wpa_supplicant_emit_changed (skeleton);
}

static void
_wpa_supplicant_on_signal_interface_added (
    WPASupplicant *object,
    const gchar *arg_path,
    GVariant *arg_properties)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(o@a{sv})",
                   arg_path,
                   arg_properties));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "fi.w1.wpa_supplicant1", "InterfaceAdded",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void
_wpa_supplicant_on_signal_interface_removed (
    WPASupplicant *object,
    const gchar *arg_path)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(o)",
                   arg_path));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "fi.w1.wpa_supplicant1", "InterfaceRemoved",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void wpa_supplicant_skeleton_iface_init (WPASupplicantIface *iface);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (WPASupplicantSkeleton, wpa_supplicant_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_ADD_PRIVATE (WPASupplicantSkeleton)
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_SUPPLICANT, wpa_supplicant_skeleton_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (WPASupplicantSkeleton, wpa_supplicant_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_SUPPLICANT, wpa_supplicant_skeleton_iface_init))

#endif
static void
wpa_supplicant_skeleton_finalize (GObject *object)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (object);
  guint n;
  for (n = 0; n < 1; n++)
    g_value_unset (&skeleton->priv->properties[n]);
  g_free (skeleton->priv->properties);
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  /* coverity[missing_lock : SUPPRESS] */
  g_clear_pointer (&skeleton->priv->changed_properties_idle_source, g_source_destroy);
#else
  if (skeleton->priv->changed_properties_idle_source != NULL)
    g_source_destroy (skeleton->priv->changed_properties_idle_source);
skeleton->priv->changed_properties_idle_source = NULL;
#endif
  g_main_context_unref (skeleton->priv->context);
  g_mutex_clear (&skeleton->priv->lock);
  G_OBJECT_CLASS (wpa_supplicant_skeleton_parent_class)->finalize (object);
}

static void
wpa_supplicant_skeleton_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 1);
  g_mutex_lock (&skeleton->priv->lock);
  g_value_copy (&skeleton->priv->properties[prop_id - 1], value);
  g_mutex_unlock (&skeleton->priv->lock);
}

static gboolean
_wpa_supplicant_emit_changed (gpointer user_data)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (user_data);
  GList *l;
  GVariantBuilder builder;
  GVariantBuilder invalidated_builder;
  guint num_changes;

  g_mutex_lock (&skeleton->priv->lock);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init_static (&invalidated_builder, G_VARIANT_TYPE ("as"));
#else
  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init (&invalidated_builder, G_VARIANT_TYPE ("as"));
#endif
  for (l = skeleton->priv->changed_properties, num_changes = 0; l != NULL; l = l->next)
    {
      ChangedProperty *cp = l->data;
      GVariant *variant;
      const GValue *cur_value;

      cur_value = &skeleton->priv->properties[cp->prop_id - 1];
      if (!_g_value_equal (cur_value, &cp->orig_value))
        {
          variant = g_dbus_gvalue_to_gvariant (cur_value, G_VARIANT_TYPE (cp->info->parent_struct.signature));
          g_variant_builder_add (&builder, "{sv}", cp->info->parent_struct.name, variant);
          g_variant_unref (variant);
          num_changes++;
        }
    }
  if (num_changes > 0)
    {
      GList *connections, *ll;
      GVariant *signal_variant;
      signal_variant = g_variant_ref_sink (g_variant_new ("(sa{sv}as)", "fi.w1.wpa_supplicant1",
                                           &builder, &invalidated_builder));
      connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));
      for (ll = connections; ll != NULL; ll = ll->next)
        {
          GDBusConnection *connection = ll->data;

          g_dbus_connection_emit_signal (connection,
                                         NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)),
                                         "org.freedesktop.DBus.Properties",
                                         "PropertiesChanged",
                                         signal_variant,
                                         NULL);
        }
      g_variant_unref (signal_variant);
      g_list_free_full (connections, g_object_unref);
    }
  else
    {
      g_variant_builder_clear (&builder);
      g_variant_builder_clear (&invalidated_builder);
    }
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  skeleton->priv->changed_properties = NULL;
  skeleton->priv->changed_properties_idle_source = NULL;
  g_mutex_unlock (&skeleton->priv->lock);
  return FALSE;
}

static void
_wpa_supplicant_schedule_emit_changed (WPASupplicantSkeleton *skeleton, const _ExtendedGDBusPropertyInfo *info, guint prop_id, const GValue *orig_value)
{
  ChangedProperty *cp;
  GList *l;
  cp = NULL;
  for (l = skeleton->priv->changed_properties; l != NULL; l = l->next)
    {
      ChangedProperty *i_cp = l->data;
      if (i_cp->info == info)
        {
          cp = i_cp;
          break;
        }
    }
  if (cp == NULL)
    {
      cp = g_new0 (ChangedProperty, 1);
      cp->prop_id = prop_id;
      cp->info = info;
      skeleton->priv->changed_properties = g_list_prepend (skeleton->priv->changed_properties, cp);
      g_value_init (&cp->orig_value, G_VALUE_TYPE (orig_value));
      g_value_copy (orig_value, &cp->orig_value);
    }
}

static void
wpa_supplicant_skeleton_notify (GObject      *object,
  GParamSpec *pspec G_GNUC_UNUSED)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (object);
  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties != NULL &&
      skeleton->priv->changed_properties_idle_source == NULL)
    {
      skeleton->priv->changed_properties_idle_source = g_idle_source_new ();
      g_source_set_priority (skeleton->priv->changed_properties_idle_source, G_PRIORITY_DEFAULT);
      g_source_set_callback (skeleton->priv->changed_properties_idle_source, _wpa_supplicant_emit_changed, g_object_ref (skeleton), (GDestroyNotify) g_object_unref);
      g_source_set_name (skeleton->priv->changed_properties_idle_source, "[generated] _wpa_supplicant_emit_changed");
      g_source_attach (skeleton->priv->changed_properties_idle_source, skeleton->priv->context);
      g_source_unref (skeleton->priv->changed_properties_idle_source);
    }
  g_mutex_unlock (&skeleton->priv->lock);
}

static void
wpa_supplicant_skeleton_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec)
{
  const _ExtendedGDBusPropertyInfo *info;
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 1);
  info = (const _ExtendedGDBusPropertyInfo *) _wpa_supplicant_property_info_pointers[prop_id - 1];
  g_mutex_lock (&skeleton->priv->lock);
  g_object_freeze_notify (object);
  if (!_g_value_equal (value, &skeleton->priv->properties[prop_id - 1]))
    {
      if (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)) != NULL &&
          info->emits_changed_signal)
        _wpa_supplicant_schedule_emit_changed (skeleton, info, prop_id, &skeleton->priv->properties[prop_id - 1]);
      g_value_copy (value, &skeleton->priv->properties[prop_id - 1]);
      g_object_notify_by_pspec (object, pspec);
    }
  g_mutex_unlock (&skeleton->priv->lock);
  g_object_thaw_notify (object);
}

static void
wpa_supplicant_skeleton_init (WPASupplicantSkeleton *skeleton)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  skeleton->priv = wpa_supplicant_skeleton_get_instance_private (skeleton);
#else
  skeleton->priv = G_TYPE_INSTANCE_GET_PRIVATE (skeleton, TYPE_WPA_SUPPLICANT_SKELETON, WPASupplicantSkeletonPrivate);
#endif

  g_mutex_init (&skeleton->priv->lock);
  skeleton->priv->context = g_main_context_ref_thread_default ();
  skeleton->priv->properties = g_new0 (GValue, 1);
  g_value_init (&skeleton->priv->properties[0], G_TYPE_STRV);
}

static const gchar *const *
wpa_supplicant_skeleton_get_interfaces (WPASupplicant *object)
{
  WPASupplicantSkeleton *skeleton = WPA_SUPPLICANT_SKELETON (object);
  const gchar *const *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_boxed (&(skeleton->priv->properties[0]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static void
wpa_supplicant_skeleton_class_init (WPASupplicantSkeletonClass *klass)
{
  GObjectClass *gobject_class;
  GDBusInterfaceSkeletonClass *skeleton_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = wpa_supplicant_skeleton_finalize;
  gobject_class->get_property = wpa_supplicant_skeleton_get_property;
  gobject_class->set_property = wpa_supplicant_skeleton_set_property;
  gobject_class->notify       = wpa_supplicant_skeleton_notify;


  wpa_supplicant_override_properties (gobject_class, 1);

  skeleton_class = G_DBUS_INTERFACE_SKELETON_CLASS (klass);
  skeleton_class->get_info = wpa_supplicant_skeleton_dbus_interface_get_info;
  skeleton_class->get_properties = wpa_supplicant_skeleton_dbus_interface_get_properties;
  skeleton_class->flush = wpa_supplicant_skeleton_dbus_interface_flush;
  skeleton_class->get_vtable = wpa_supplicant_skeleton_dbus_interface_get_vtable;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (WPASupplicantSkeletonPrivate));
#endif
}

static void
wpa_supplicant_skeleton_iface_init (WPASupplicantIface *iface)
{
  iface->interface_added = _wpa_supplicant_on_signal_interface_added;
  iface->interface_removed = _wpa_supplicant_on_signal_interface_removed;
  iface->get_interfaces = wpa_supplicant_skeleton_get_interfaces;
}

/**
 * wpa_supplicant_skeleton_new:
 *
 * Creates a skeleton object for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1.top_of_page">fi.w1.wpa_supplicant1</link>.
 *
 * Returns: (transfer full) (type WPASupplicantSkeleton): The skeleton object.
 */
WPASupplicant *
wpa_supplicant_skeleton_new (void)
{
  return WPA_SUPPLICANT (g_object_new (TYPE_WPA_SUPPLICANT_SKELETON, NULL));
}

/* ------------------------------------------------------------------------
 * Code for interface fi.w1.wpa_supplicant1.Interface
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:WPAInterface
 * @title: WPAInterface
 * @short_description: Generated C code for the fi.w1.wpa_supplicant1.Interface D-Bus interface
 *
 * This section contains code for working with the <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-Interface.top_of_page">fi.w1.wpa_supplicant1.Interface</link> D-Bus interface in C.
 */

enum
{
  _WPA_INTERFACE_BSS_ADDED,
  _WPA_INTERFACE_BSS_REMOVED,
  _WPA_INTERFACE_SCAN_DONE,
};

static unsigned _WPA_INTERFACE_SIGNALS[3] = { 0 };

/* ---- Introspection data for fi.w1.wpa_supplicant1.Interface ---- */

static const _ExtendedGDBusArgInfo _wpa_interface_method_info_scan_IN_ARG_args =
{
  {
    -1,
    (gchar *) "args",
    (gchar *) "a{sv}",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _wpa_interface_method_info_scan_IN_ARG_pointers[] =
{
  &_wpa_interface_method_info_scan_IN_ARG_args.parent_struct,
  NULL
};

static const _ExtendedGDBusMethodInfo _wpa_interface_method_info_scan =
{
  {
    -1,
    (gchar *) "Scan",
    (GDBusArgInfo **) &_wpa_interface_method_info_scan_IN_ARG_pointers,
    NULL,
    NULL
  },
  "handle-scan",
  FALSE
};

static const GDBusMethodInfo * const _wpa_interface_method_info_pointers[] =
{
  &_wpa_interface_method_info_scan.parent_struct,
  NULL
};

static const _ExtendedGDBusArgInfo _wpa_interface_signal_info_bss_added_ARG_path =
{
  {
    -1,
    (gchar *) "path",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const _ExtendedGDBusArgInfo _wpa_interface_signal_info_bss_added_ARG_properties =
{
  {
    -1,
    (gchar *) "properties",
    (gchar *) "a{sv}",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _wpa_interface_signal_info_bss_added_ARG_pointers[] =
{
  &_wpa_interface_signal_info_bss_added_ARG_path.parent_struct,
  &_wpa_interface_signal_info_bss_added_ARG_properties.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _wpa_interface_signal_info_bss_added =
{
  {
    -1,
    (gchar *) "BSSAdded",
    (GDBusArgInfo **) &_wpa_interface_signal_info_bss_added_ARG_pointers,
    NULL
  },
  "bss-added"
};

static const _ExtendedGDBusArgInfo _wpa_interface_signal_info_bss_removed_ARG_path =
{
  {
    -1,
    (gchar *) "path",
    (gchar *) "o",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _wpa_interface_signal_info_bss_removed_ARG_pointers[] =
{
  &_wpa_interface_signal_info_bss_removed_ARG_path.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _wpa_interface_signal_info_bss_removed =
{
  {
    -1,
    (gchar *) "BSSRemoved",
    (GDBusArgInfo **) &_wpa_interface_signal_info_bss_removed_ARG_pointers,
    NULL
  },
  "bss-removed"
};

static const _ExtendedGDBusArgInfo _wpa_interface_signal_info_scan_done_ARG_success =
{
  {
    -1,
    (gchar *) "success",
    (gchar *) "b",
    NULL
  },
  FALSE
};

static const GDBusArgInfo * const _wpa_interface_signal_info_scan_done_ARG_pointers[] =
{
  &_wpa_interface_signal_info_scan_done_ARG_success.parent_struct,
  NULL
};

static const _ExtendedGDBusSignalInfo _wpa_interface_signal_info_scan_done =
{
  {
    -1,
    (gchar *) "ScanDone",
    (GDBusArgInfo **) &_wpa_interface_signal_info_scan_done_ARG_pointers,
    NULL
  },
  "scan-done"
};

static const GDBusSignalInfo * const _wpa_interface_signal_info_pointers[] =
{
  &_wpa_interface_signal_info_bss_added.parent_struct,
  &_wpa_interface_signal_info_bss_removed.parent_struct,
  &_wpa_interface_signal_info_scan_done.parent_struct,
  NULL
};

static const _ExtendedGDBusPropertyInfo _wpa_interface_property_info_state =
{
  {
    -1,
    (gchar *) "State",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "state",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _wpa_interface_property_info_ifname =
{
  {
    -1,
    (gchar *) "Ifname",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "ifname",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _wpa_interface_property_info_driver =
{
  {
    -1,
    (gchar *) "Driver",
    (gchar *) "s",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "driver",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _wpa_interface_property_info_bsss =
{
  {
    -1,
    (gchar *) "BSSs",
    (gchar *) "ao",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "bsss",
  FALSE,
  TRUE
};

static const GDBusPropertyInfo * const _wpa_interface_property_info_pointers[] =
{
  &_wpa_interface_property_info_state.parent_struct,
  &_wpa_interface_property_info_ifname.parent_struct,
  &_wpa_interface_property_info_driver.parent_struct,
  &_wpa_interface_property_info_bsss.parent_struct,
  NULL
};

static const _ExtendedGDBusInterfaceInfo _wpa_interface_interface_info =
{
  {
    -1,
    (gchar *) "fi.w1.wpa_supplicant1.Interface",
    (GDBusMethodInfo **) &_wpa_interface_method_info_pointers,
    (GDBusSignalInfo **) &_wpa_interface_signal_info_pointers,
    (GDBusPropertyInfo **) &_wpa_interface_property_info_pointers,
    NULL
  },
  "wpa-interface",
};


/**
 * wpa_interface_interface_info:
 *
 * Gets a machine-readable description of the <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-Interface.top_of_page">fi.w1.wpa_supplicant1.Interface</link> D-Bus interface.
 *
 * Returns: (transfer none): A #GDBusInterfaceInfo. Do not free.
 */
GDBusInterfaceInfo *
wpa_interface_interface_info (void)
{
  return (GDBusInterfaceInfo *) &_wpa_interface_interface_info.parent_struct;
}

/**
 * wpa_interface_override_properties:
 * @klass: The class structure for a #GObject derived class.
 * @property_id_begin: The property id to assign to the first overridden property.
 *
 * Overrides all #GObject properties in the #WPAInterface interface for a concrete class.
 * The properties are overridden in the order they are defined.
 *
 * Returns: The last property id.
 */
guint
wpa_interface_override_properties (GObjectClass *klass, guint property_id_begin)
{
  g_object_class_override_property (klass, property_id_begin++, "state");
  g_object_class_override_property (klass, property_id_begin++, "ifname");
  g_object_class_override_property (klass, property_id_begin++, "driver");
  g_object_class_override_property (klass, property_id_begin++, "bsss");
  return property_id_begin - 1;
}


inline static void
wpa_interface_signal_marshal_bss_added (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_VOID__STRING_VARIANT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
wpa_interface_signal_marshal_bss_removed (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__STRING (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
wpa_interface_signal_marshal_scan_done (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  g_cclosure_marshal_VOID__BOOLEAN (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}

inline static void
wpa_interface_method_marshal_scan (
    GClosure     *closure,
    GValue       *return_value,
    unsigned int  n_param_values,
    const GValue *param_values,
    void         *invocation_hint,
    void         *marshal_data)
{
  _g_dbus_codegen_marshal_BOOLEAN__OBJECT_VARIANT (closure,
    return_value, n_param_values, param_values, invocation_hint, marshal_data);
}


/**
 * WPAInterface:
 *
 * Abstract interface type for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-Interface.top_of_page">fi.w1.wpa_supplicant1.Interface</link>.
 */

/**
 * WPAInterfaceIface:
 * @parent_iface: The parent interface.
 * @handle_scan: Handler for the #WPAInterface::handle-scan signal.
 * @get_bsss: Getter for the #WPAInterface:bsss property.
 * @get_driver: Getter for the #WPAInterface:driver property.
 * @get_ifname: Getter for the #WPAInterface:ifname property.
 * @get_state: Getter for the #WPAInterface:state property.
 * @bss_added: Handler for the #WPAInterface::bss-added signal.
 * @bss_removed: Handler for the #WPAInterface::bss-removed signal.
 * @scan_done: Handler for the #WPAInterface::scan-done signal.
 *
 * Virtual table for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-Interface.top_of_page">fi.w1.wpa_supplicant1.Interface</link>.
 */

typedef WPAInterfaceIface WPAInterfaceInterface;
G_DEFINE_INTERFACE (WPAInterface, wpa_interface, G_TYPE_OBJECT)

static void
wpa_interface_default_init (WPAInterfaceIface *iface)
{
  /* GObject signals for incoming D-Bus method calls: */
  /**
   * WPAInterface::handle-scan:
   * @object: A #WPAInterface.
   * @invocation: A #GDBusMethodInvocation.
   * @arg_args: Argument passed by remote caller.
   *
   * Signal emitted when a remote caller is invoking the <link linkend="gdbus-method-fi-w1-wpa_supplicant1-Interface.Scan">Scan()</link> D-Bus method.
   *
   * If a signal handler returns %TRUE, it means the signal handler will handle the invocation (e.g. take a reference to @invocation and eventually call wpa_interface_complete_scan() or e.g. g_dbus_method_invocation_return_error() on it) and no other signal handlers will run. If no signal handler handles the invocation, the %G_DBUS_ERROR_UNKNOWN_METHOD error is returned.
   *
   * Returns: %G_DBUS_METHOD_INVOCATION_HANDLED or %TRUE if the invocation was handled, %G_DBUS_METHOD_INVOCATION_UNHANDLED or %FALSE to let other signal handlers run.
   */
  g_signal_new ("handle-scan",
    G_TYPE_FROM_INTERFACE (iface),
    G_SIGNAL_RUN_LAST,
    G_STRUCT_OFFSET (WPAInterfaceIface, handle_scan),
    g_signal_accumulator_true_handled,
    NULL,
      wpa_interface_method_marshal_scan,
    G_TYPE_BOOLEAN,
    2,
    G_TYPE_DBUS_METHOD_INVOCATION, G_TYPE_VARIANT);

  /* GObject signals for received D-Bus signals: */
  /**
   * WPAInterface::bss-added:
   * @object: A #WPAInterface.
   * @arg_path: Argument.
   * @arg_properties: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-fi-w1-wpa_supplicant1-Interface.BSSAdded">"BSSAdded"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _WPA_INTERFACE_SIGNALS[_WPA_INTERFACE_BSS_ADDED] =
    g_signal_new ("bss-added",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (WPAInterfaceIface, bss_added),
      NULL,
      NULL,
      wpa_interface_signal_marshal_bss_added,
      G_TYPE_NONE,
      2, G_TYPE_STRING, G_TYPE_VARIANT);

  /**
   * WPAInterface::bss-removed:
   * @object: A #WPAInterface.
   * @arg_path: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-fi-w1-wpa_supplicant1-Interface.BSSRemoved">"BSSRemoved"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _WPA_INTERFACE_SIGNALS[_WPA_INTERFACE_BSS_REMOVED] =
    g_signal_new ("bss-removed",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (WPAInterfaceIface, bss_removed),
      NULL,
      NULL,
      wpa_interface_signal_marshal_bss_removed,
      G_TYPE_NONE,
      1, G_TYPE_STRING);

  /**
   * WPAInterface::scan-done:
   * @object: A #WPAInterface.
   * @arg_success: Argument.
   *
   * On the client-side, this signal is emitted whenever the D-Bus signal <link linkend="gdbus-signal-fi-w1-wpa_supplicant1-Interface.ScanDone">"ScanDone"</link> is received.
   *
   * On the service-side, this signal can be used with e.g. g_signal_emit_by_name() to make the object emit the D-Bus signal.
   */
  _WPA_INTERFACE_SIGNALS[_WPA_INTERFACE_SCAN_DONE] =
    g_signal_new ("scan-done",
      G_TYPE_FROM_INTERFACE (iface),
      G_SIGNAL_RUN_LAST,
      G_STRUCT_OFFSET (WPAInterfaceIface, scan_done),
      NULL,
      NULL,
      wpa_interface_signal_marshal_scan_done,
      G_TYPE_NONE,
      1, G_TYPE_BOOLEAN);

  /* GObject properties for D-Bus properties: */
  /**
   * WPAInterface:state:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.State">"State"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("state", "State", "State", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * WPAInterface:ifname:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.Ifname">"Ifname"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("ifname", "Ifname", "Ifname", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * WPAInterface:driver:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.Driver">"Driver"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_string ("driver", "Driver", "Driver", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * WPAInterface:bsss:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.BSSs">"BSSs"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_boxed ("bsss", "BSSs", "BSSs", G_TYPE_STRV, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
}

/**
 * wpa_interface_get_state: (skip)
 * @object: A #WPAInterface.
 *
 * Gets the value of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.State">"State"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use wpa_interface_dup_state() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
wpa_interface_get_state (WPAInterface *object)
{
  g_return_val_if_fail (IS_WPA_INTERFACE (object), NULL);

  return WPA_INTERFACE_GET_IFACE (object)->get_state (object);
}

/**
 * wpa_interface_dup_state: (skip)
 * @object: A #WPAInterface.
 *
 * Gets a copy of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.State">"State"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
wpa_interface_dup_state (WPAInterface *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "state", &value, NULL);
  return value;
}

/**
 * wpa_interface_set_state: (skip)
 * @object: A #WPAInterface.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.State">"State"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
wpa_interface_set_state (WPAInterface *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "state", value, NULL);
}

/**
 * wpa_interface_get_ifname: (skip)
 * @object: A #WPAInterface.
 *
 * Gets the value of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.Ifname">"Ifname"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use wpa_interface_dup_ifname() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
wpa_interface_get_ifname (WPAInterface *object)
{
  g_return_val_if_fail (IS_WPA_INTERFACE (object), NULL);

  return WPA_INTERFACE_GET_IFACE (object)->get_ifname (object);
}

/**
 * wpa_interface_dup_ifname: (skip)
 * @object: A #WPAInterface.
 *
 * Gets a copy of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.Ifname">"Ifname"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
wpa_interface_dup_ifname (WPAInterface *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "ifname", &value, NULL);
  return value;
}

/**
 * wpa_interface_set_ifname: (skip)
 * @object: A #WPAInterface.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.Ifname">"Ifname"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
wpa_interface_set_ifname (WPAInterface *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "ifname", value, NULL);
}

/**
 * wpa_interface_get_driver: (skip)
 * @object: A #WPAInterface.
 *
 * Gets the value of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.Driver">"Driver"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use wpa_interface_dup_driver() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *
wpa_interface_get_driver (WPAInterface *object)
{
  g_return_val_if_fail (IS_WPA_INTERFACE (object), NULL);

  return WPA_INTERFACE_GET_IFACE (object)->get_driver (object);
}

/**
 * wpa_interface_dup_driver: (skip)
 * @object: A #WPAInterface.
 *
 * Gets a copy of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.Driver">"Driver"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_free().
 */
gchar *
wpa_interface_dup_driver (WPAInterface *object)
{
  gchar *value;
  g_object_get (G_OBJECT (object), "driver", &value, NULL);
  return value;
}

/**
 * wpa_interface_set_driver: (skip)
 * @object: A #WPAInterface.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.Driver">"Driver"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
wpa_interface_set_driver (WPAInterface *object, const gchar *value)
{
  g_object_set (G_OBJECT (object), "driver", value, NULL);
}

/**
 * wpa_interface_get_bsss: (skip)
 * @object: A #WPAInterface.
 *
 * Gets the value of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.BSSs">"BSSs"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use wpa_interface_dup_bsss() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
const gchar *const *
wpa_interface_get_bsss (WPAInterface *object)
{
  g_return_val_if_fail (IS_WPA_INTERFACE (object), NULL);

  return WPA_INTERFACE_GET_IFACE (object)->get_bsss (object);
}

/**
 * wpa_interface_dup_bsss: (skip)
 * @object: A #WPAInterface.
 *
 * Gets a copy of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.BSSs">"BSSs"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_strfreev().
 */
gchar **
wpa_interface_dup_bsss (WPAInterface *object)
{
  gchar **value;
  g_object_get (G_OBJECT (object), "bsss", &value, NULL);
  return value;
}

/**
 * wpa_interface_set_bsss: (skip)
 * @object: A #WPAInterface.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-Interface.BSSs">"BSSs"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
wpa_interface_set_bsss (WPAInterface *object, const gchar *const *value)
{
  g_object_set (G_OBJECT (object), "bsss", value, NULL);
}

/**
 * wpa_interface_emit_bss_added:
 * @object: A #WPAInterface.
 * @arg_path: Argument to pass with the signal.
 * @arg_properties: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-fi-w1-wpa_supplicant1-Interface.BSSAdded">"BSSAdded"</link> D-Bus signal.
 */
void
wpa_interface_emit_bss_added (
    WPAInterface *object,
    const gchar *arg_path,
    GVariant *arg_properties)
{
  g_signal_emit (object, _WPA_INTERFACE_SIGNALS[_WPA_INTERFACE_BSS_ADDED], 0, arg_path, arg_properties);
}

/**
 * wpa_interface_emit_bss_removed:
 * @object: A #WPAInterface.
 * @arg_path: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-fi-w1-wpa_supplicant1-Interface.BSSRemoved">"BSSRemoved"</link> D-Bus signal.
 */
void
wpa_interface_emit_bss_removed (
    WPAInterface *object,
    const gchar *arg_path)
{
  g_signal_emit (object, _WPA_INTERFACE_SIGNALS[_WPA_INTERFACE_BSS_REMOVED], 0, arg_path);
}

/**
 * wpa_interface_emit_scan_done:
 * @object: A #WPAInterface.
 * @arg_success: Argument to pass with the signal.
 *
 * Emits the <link linkend="gdbus-signal-fi-w1-wpa_supplicant1-Interface.ScanDone">"ScanDone"</link> D-Bus signal.
 */
void
wpa_interface_emit_scan_done (
    WPAInterface *object,
    gboolean arg_success)
{
  g_signal_emit (object, _WPA_INTERFACE_SIGNALS[_WPA_INTERFACE_SCAN_DONE], 0, arg_success);
}

/**
 * wpa_interface_call_scan:
 * @proxy: A #WPAInterfaceProxy.
 * @arg_args: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied or %NULL.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously invokes the <link linkend="gdbus-method-fi-w1-wpa_supplicant1-Interface.Scan">Scan()</link> D-Bus method on @proxy.
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call wpa_interface_call_scan_finish() to get the result of the operation.
 *
 * See wpa_interface_call_scan_sync() for the synchronous, blocking version of this method.
 */
void
wpa_interface_call_scan (
    WPAInterface *proxy,
    GVariant *arg_args,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
  g_dbus_proxy_call (G_DBUS_PROXY (proxy),
    "Scan",
    g_variant_new ("(@a{sv})",
                   arg_args),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    callback,
    user_data);
}

/**
 * wpa_interface_call_scan_finish:
 * @proxy: A #WPAInterfaceProxy.
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to wpa_interface_call_scan().
 * @error: Return location for error or %NULL.
 *
 * Finishes an operation started with wpa_interface_call_scan().
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
wpa_interface_call_scan_finish (
    WPAInterface *proxy,
    GAsyncResult *res,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_finish (G_DBUS_PROXY (proxy), res, error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * wpa_interface_call_scan_sync:
 * @proxy: A #WPAInterfaceProxy.
 * @arg_args: Argument to pass with the method invocation.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL.
 *
 * Synchronously invokes the <link linkend="gdbus-method-fi-w1-wpa_supplicant1-Interface.Scan">Scan()</link> D-Bus method on @proxy. The calling thread is blocked until a reply is received.
 *
 * See wpa_interface_call_scan() for the asynchronous version of this method.
 *
 * Returns: (skip): %TRUE if the call succeeded, %FALSE if @error is set.
 */
gboolean
wpa_interface_call_scan_sync (
    WPAInterface *proxy,
    GVariant *arg_args,
    GCancellable *cancellable,
    GError **error)
{
  GVariant *_ret;
  _ret = g_dbus_proxy_call_sync (G_DBUS_PROXY (proxy),
    "Scan",
    g_variant_new ("(@a{sv})",
                   arg_args),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    cancellable,
    error);
  if (_ret == NULL)
    goto _out;
  g_variant_get (_ret,
                 "()");
  g_variant_unref (_ret);
_out:
  return _ret != NULL;
}

/**
 * wpa_interface_complete_scan:
 * @object: A #WPAInterface.
 * @invocation: (transfer full): A #GDBusMethodInvocation.
 *
 * Helper function used in service implementations to finish handling invocations of the <link linkend="gdbus-method-fi-w1-wpa_supplicant1-Interface.Scan">Scan()</link> D-Bus method. If you instead want to finish handling an invocation by returning an error, use g_dbus_method_invocation_return_error() or similar.
 *
 * This method will free @invocation, you cannot use it afterwards.
 */
void
wpa_interface_complete_scan (
    WPAInterface *object G_GNUC_UNUSED,
    GDBusMethodInvocation *invocation)
{
  g_dbus_method_invocation_return_value (invocation,
    g_variant_new ("()"));
}

/* ------------------------------------------------------------------------ */

/**
 * WPAInterfaceProxy:
 *
 * The #WPAInterfaceProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * WPAInterfaceProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #WPAInterfaceProxy.
 */

struct _WPAInterfaceProxyPrivate
{
  GData *qdata;
};

static void wpa_interface_proxy_iface_init (WPAInterfaceIface *iface);

#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (WPAInterfaceProxy, wpa_interface_proxy, G_TYPE_DBUS_PROXY,
                         G_ADD_PRIVATE (WPAInterfaceProxy)
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_INTERFACE, wpa_interface_proxy_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (WPAInterfaceProxy, wpa_interface_proxy, G_TYPE_DBUS_PROXY,
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_INTERFACE, wpa_interface_proxy_iface_init))

#endif
static void
wpa_interface_proxy_finalize (GObject *object)
{
  WPAInterfaceProxy *proxy = WPA_INTERFACE_PROXY (object);
  g_datalist_clear (&proxy->priv->qdata);
  G_OBJECT_CLASS (wpa_interface_proxy_parent_class)->finalize (object);
}

static void
wpa_interface_proxy_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 4);
  info = (const _ExtendedGDBusPropertyInfo *) _wpa_interface_property_info_pointers[prop_id - 1];
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (object), info->parent_struct.name);
  if (info->use_gvariant)
    {
      g_value_set_variant (value, variant);
    }
  else
    {
      if (variant != NULL)
        g_dbus_gvariant_to_gvalue (variant, value);
    }
  if (variant != NULL)
    g_variant_unref (variant);
}

static void
wpa_interface_proxy_set_property_cb (GDBusProxy *proxy,
  GAsyncResult *res,
  gpointer      user_data)
{
  const _ExtendedGDBusPropertyInfo *info = user_data;
  GError *error;
  GVariant *_ret;
  error = NULL;
  _ret = g_dbus_proxy_call_finish (proxy, res, &error);
  if (!_ret)
    {
      g_warning ("Error setting property '%s' on interface fi.w1.wpa_supplicant1.Interface: %s (%s, %d)",
                 info->parent_struct.name, 
                 error->message, g_quark_to_string (error->domain), error->code);
      g_error_free (error);
    }
  else
    {
      g_variant_unref (_ret);
    }
}

static void
wpa_interface_proxy_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 4);
  info = (const _ExtendedGDBusPropertyInfo *) _wpa_interface_property_info_pointers[prop_id - 1];
  variant = g_dbus_gvalue_to_gvariant (value, G_VARIANT_TYPE (info->parent_struct.signature));
  g_dbus_proxy_call (G_DBUS_PROXY (object),
    "org.freedesktop.DBus.Properties.Set",
    g_variant_new ("(ssv)", "fi.w1.wpa_supplicant1.Interface", info->parent_struct.name, variant),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    NULL, (GAsyncReadyCallback) wpa_interface_proxy_set_property_cb, (GDBusPropertyInfo *) &info->parent_struct);
  g_variant_unref (variant);
}

static void
wpa_interface_proxy_g_signal (GDBusProxy *proxy,
  const gchar *sender_name G_GNUC_UNUSED,
  const gchar *signal_name,
  GVariant *parameters)
{
  _ExtendedGDBusSignalInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  gsize n;
  guint signal_id;
  info = (_ExtendedGDBusSignalInfo *) g_dbus_interface_info_lookup_signal ((GDBusInterfaceInfo *) &_wpa_interface_interface_info.parent_struct, signal_name);
  if (info == NULL)
    return;
  num_params = g_variant_n_children (parameters);
  paramv = g_new0 (GValue, num_params + 1);
  g_value_init (&paramv[0], TYPE_WPA_INTERFACE);
  g_value_set_object (&paramv[0], proxy);
  g_variant_iter_init (&iter, parameters);
  n = 1;
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.args[n - 1];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_WPA_INTERFACE);
  g_signal_emitv (paramv, signal_id, 0, NULL);
  for (n = 0; n < num_params + 1; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static void
wpa_interface_proxy_g_properties_changed (GDBusProxy *_proxy,
  GVariant *changed_properties,
  const gchar *const *invalidated_properties)
{
  WPAInterfaceProxy *proxy = WPA_INTERFACE_PROXY (_proxy);
  guint n;
  const gchar *key;
  GVariantIter *iter;
  _ExtendedGDBusPropertyInfo *info;
  g_variant_get (changed_properties, "a{sv}", &iter);
  while (g_variant_iter_next (iter, "{&sv}", &key, NULL))
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_interface_interface_info.parent_struct, key);
      g_datalist_remove_data (&proxy->priv->qdata, key);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
  g_variant_iter_free (iter);
  for (n = 0; invalidated_properties[n] != NULL; n++)
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_interface_interface_info.parent_struct, invalidated_properties[n]);
      g_datalist_remove_data (&proxy->priv->qdata, invalidated_properties[n]);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
}

static const gchar *
wpa_interface_proxy_get_state (WPAInterface *object)
{
  WPAInterfaceProxy *proxy = WPA_INTERFACE_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "State");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *
wpa_interface_proxy_get_ifname (WPAInterface *object)
{
  WPAInterfaceProxy *proxy = WPA_INTERFACE_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Ifname");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *
wpa_interface_proxy_get_driver (WPAInterface *object)
{
  WPAInterfaceProxy *proxy = WPA_INTERFACE_PROXY (object);
  GVariant *variant;
  const gchar *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Driver");
  if (variant != NULL)
    {
      value = g_variant_get_string (variant, NULL);
      g_variant_unref (variant);
    }
  return value;
}

static const gchar *const *
wpa_interface_proxy_get_bsss (WPAInterface *object)
{
  WPAInterfaceProxy *proxy = WPA_INTERFACE_PROXY (object);
  GVariant *variant;
  const gchar *const *value = NULL;
  value = g_datalist_get_data (&proxy->priv->qdata, "BSSs");
  if (value != NULL)
    return value;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "BSSs");
  if (variant != NULL)
    {
      value = g_variant_get_objv (variant, NULL);
      g_datalist_set_data_full (&proxy->priv->qdata, "BSSs", (gpointer) value, g_free);
      g_variant_unref (variant);
    }
  return value;
}

static void
wpa_interface_proxy_init (WPAInterfaceProxy *proxy)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  proxy->priv = wpa_interface_proxy_get_instance_private (proxy);
#else
  proxy->priv = G_TYPE_INSTANCE_GET_PRIVATE (proxy, TYPE_WPA_INTERFACE_PROXY, WPAInterfaceProxyPrivate);
#endif

  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (proxy), wpa_interface_interface_info ());
}

static void
wpa_interface_proxy_class_init (WPAInterfaceProxyClass *klass)
{
  GObjectClass *gobject_class;
  GDBusProxyClass *proxy_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize     = wpa_interface_proxy_finalize;
  gobject_class->get_property = wpa_interface_proxy_get_property;
  gobject_class->set_property = wpa_interface_proxy_set_property;

  proxy_class = G_DBUS_PROXY_CLASS (klass);
  proxy_class->g_signal = wpa_interface_proxy_g_signal;
  proxy_class->g_properties_changed = wpa_interface_proxy_g_properties_changed;

  wpa_interface_override_properties (gobject_class, 1);

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (WPAInterfaceProxyPrivate));
#endif
}

static void
wpa_interface_proxy_iface_init (WPAInterfaceIface *iface)
{
  iface->get_state = wpa_interface_proxy_get_state;
  iface->get_ifname = wpa_interface_proxy_get_ifname;
  iface->get_driver = wpa_interface_proxy_get_driver;
  iface->get_bsss = wpa_interface_proxy_get_bsss;
}

/**
 * wpa_interface_proxy_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-Interface.top_of_page">fi.w1.wpa_supplicant1.Interface</link>. See g_dbus_proxy_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call wpa_interface_proxy_new_finish() to get the result of the operation.
 *
 * See wpa_interface_proxy_new_sync() for the synchronous, blocking version of this constructor.
 */
void
wpa_interface_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_WPA_INTERFACE_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1.Interface", NULL);
}

/**
 * wpa_interface_proxy_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to wpa_interface_proxy_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with wpa_interface_proxy_new().
 *
 * Returns: (transfer full) (type WPAInterfaceProxy): The constructed proxy object or %NULL if @error is set.
 */
WPAInterface *
wpa_interface_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return WPA_INTERFACE (ret);
  else
    return NULL;
}

/**
 * wpa_interface_proxy_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-Interface.top_of_page">fi.w1.wpa_supplicant1.Interface</link>. See g_dbus_proxy_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See wpa_interface_proxy_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type WPAInterfaceProxy): The constructed proxy object or %NULL if @error is set.
 */
WPAInterface *
wpa_interface_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_WPA_INTERFACE_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1.Interface", NULL);
  if (ret != NULL)
    return WPA_INTERFACE (ret);
  else
    return NULL;
}


/**
 * wpa_interface_proxy_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like wpa_interface_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call wpa_interface_proxy_new_for_bus_finish() to get the result of the operation.
 *
 * See wpa_interface_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
wpa_interface_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_WPA_INTERFACE_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1.Interface", NULL);
}

/**
 * wpa_interface_proxy_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to wpa_interface_proxy_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with wpa_interface_proxy_new_for_bus().
 *
 * Returns: (transfer full) (type WPAInterfaceProxy): The constructed proxy object or %NULL if @error is set.
 */
WPAInterface *
wpa_interface_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return WPA_INTERFACE (ret);
  else
    return NULL;
}

/**
 * wpa_interface_proxy_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like wpa_interface_proxy_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See wpa_interface_proxy_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type WPAInterfaceProxy): The constructed proxy object or %NULL if @error is set.
 */
WPAInterface *
wpa_interface_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_WPA_INTERFACE_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1.Interface", NULL);
  if (ret != NULL)
    return WPA_INTERFACE (ret);
  else
    return NULL;
}


/* ------------------------------------------------------------------------ */

/**
 * WPAInterfaceSkeleton:
 *
 * The #WPAInterfaceSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * WPAInterfaceSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #WPAInterfaceSkeleton.
 */

struct _WPAInterfaceSkeletonPrivate
{
  GValue *properties;
  GList *changed_properties;
  GSource *changed_properties_idle_source;
  GMainContext *context;
  GMutex lock;
};

static void
_wpa_interface_skeleton_handle_method_call (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name,
  const gchar *method_name,
  GVariant *parameters,
  GDBusMethodInvocation *invocation,
  gpointer user_data)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (user_data);
  _ExtendedGDBusMethodInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  guint num_extra;
  gsize n;
  guint signal_id;
  GValue return_value = G_VALUE_INIT;
  info = (_ExtendedGDBusMethodInfo *) g_dbus_method_invocation_get_method_info (invocation);
  g_assert (info != NULL);
  num_params = g_variant_n_children (parameters);
  num_extra = info->pass_fdlist ? 3 : 2;  paramv = g_new0 (GValue, num_params + num_extra);
  n = 0;
  g_value_init (&paramv[n], TYPE_WPA_INTERFACE);
  g_value_set_object (&paramv[n++], skeleton);
  g_value_init (&paramv[n], G_TYPE_DBUS_METHOD_INVOCATION);
  g_value_set_object (&paramv[n++], invocation);
  if (info->pass_fdlist)
    {
#ifdef G_OS_UNIX
      g_value_init (&paramv[n], G_TYPE_UNIX_FD_LIST);
      g_value_set_object (&paramv[n++], g_dbus_message_get_unix_fd_list (g_dbus_method_invocation_get_message (invocation)));
#else
      g_assert_not_reached ();
#endif
    }
  g_variant_iter_init (&iter, parameters);
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.in_args[n - num_extra];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_WPA_INTERFACE);
  g_value_init (&return_value, G_TYPE_BOOLEAN);
  g_signal_emitv (paramv, signal_id, 0, &return_value);
  if (!g_value_get_boolean (&return_value))
    g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Method %s is not implemented on interface %s", method_name, interface_name);
  g_value_unset (&return_value);
  for (n = 0; n < num_params + num_extra; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static GVariant *
_wpa_interface_skeleton_handle_get_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GError **error,
  gpointer user_data)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  GVariant *ret;
  ret = NULL;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_interface_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      g_value_init (&value, pspec->value_type);
      g_object_get_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      ret = g_dbus_gvalue_to_gvariant (&value, G_VARIANT_TYPE (info->parent_struct.signature));
      g_value_unset (&value);
    }
  return ret;
}

static gboolean
_wpa_interface_skeleton_handle_set_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GVariant *variant,
  GError **error,
  gpointer user_data)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  gboolean ret;
  ret = FALSE;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_interface_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      if (info->use_gvariant)
        g_value_set_variant (&value, variant);
      else
        g_dbus_gvariant_to_gvalue (variant, &value);
      g_object_set_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      g_value_unset (&value);
      ret = TRUE;
    }
  return ret;
}

static const GDBusInterfaceVTable _wpa_interface_skeleton_vtable =
{
  _wpa_interface_skeleton_handle_method_call,
  _wpa_interface_skeleton_handle_get_property,
  _wpa_interface_skeleton_handle_set_property,
  {NULL}
};

static GDBusInterfaceInfo *
wpa_interface_skeleton_dbus_interface_get_info (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return wpa_interface_interface_info ();
}

static GDBusInterfaceVTable *
wpa_interface_skeleton_dbus_interface_get_vtable (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return (GDBusInterfaceVTable *) &_wpa_interface_skeleton_vtable;
}

static GVariant *
wpa_interface_skeleton_dbus_interface_get_properties (GDBusInterfaceSkeleton *_skeleton)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (_skeleton);

  GVariantBuilder builder;
  guint n;
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
#else
  g_variant_builder_init(&builder, G_VARIANT_TYPE ("a{sv}"));
#endif
  if (_wpa_interface_interface_info.parent_struct.properties == NULL)
    goto out;
  for (n = 0; _wpa_interface_interface_info.parent_struct.properties[n] != NULL; n++)
    {
      GDBusPropertyInfo *info = _wpa_interface_interface_info.parent_struct.properties[n];
      if (info->flags & G_DBUS_PROPERTY_INFO_FLAGS_READABLE)
        {
          GVariant *value;
          value = _wpa_interface_skeleton_handle_get_property (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)), NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "fi.w1.wpa_supplicant1.Interface", info->name, NULL, skeleton);
          if (value != NULL)
            {
              g_variant_take_ref (value);
              g_variant_builder_add (&builder, "{sv}", info->name, value);
              g_variant_unref (value);
            }
        }
    }
out:
  return g_variant_builder_end (&builder);
}

static gboolean _wpa_interface_emit_changed (gpointer user_data);

static void
wpa_interface_skeleton_dbus_interface_flush (GDBusInterfaceSkeleton *_skeleton)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (_skeleton);
  gboolean emit_changed = FALSE;

  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    {
      g_source_destroy (skeleton->priv->changed_properties_idle_source);
      skeleton->priv->changed_properties_idle_source = NULL;
      emit_changed = TRUE;
    }
  g_mutex_unlock (&skeleton->priv->lock);

  if (emit_changed)
    _wpa_interface_emit_changed (skeleton);
}

static void
_wpa_interface_on_signal_bss_added (
    WPAInterface *object,
    const gchar *arg_path,
    GVariant *arg_properties)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(o@a{sv})",
                   arg_path,
                   arg_properties));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "fi.w1.wpa_supplicant1.Interface", "BSSAdded",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void
_wpa_interface_on_signal_bss_removed (
    WPAInterface *object,
    const gchar *arg_path)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(o)",
                   arg_path));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "fi.w1.wpa_supplicant1.Interface", "BSSRemoved",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void
_wpa_interface_on_signal_scan_done (
    WPAInterface *object,
    gboolean arg_success)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (object);

  GList      *connections, *l;
  GVariant   *signal_variant;
  connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));

  signal_variant = g_variant_ref_sink (g_variant_new ("(b)",
                   arg_success));
  for (l = connections; l != NULL; l = l->next)
    {
      GDBusConnection *connection = l->data;
      g_dbus_connection_emit_signal (connection,
        NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "fi.w1.wpa_supplicant1.Interface", "ScanDone",
        signal_variant, NULL);
    }
  g_variant_unref (signal_variant);
  g_list_free_full (connections, g_object_unref);
}

static void wpa_interface_skeleton_iface_init (WPAInterfaceIface *iface);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (WPAInterfaceSkeleton, wpa_interface_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_ADD_PRIVATE (WPAInterfaceSkeleton)
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_INTERFACE, wpa_interface_skeleton_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (WPAInterfaceSkeleton, wpa_interface_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_INTERFACE, wpa_interface_skeleton_iface_init))

#endif
static void
wpa_interface_skeleton_finalize (GObject *object)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (object);
  guint n;
  for (n = 0; n < 4; n++)
    g_value_unset (&skeleton->priv->properties[n]);
  g_free (skeleton->priv->properties);
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  /* coverity[missing_lock : SUPPRESS] */
  g_clear_pointer (&skeleton->priv->changed_properties_idle_source, g_source_destroy);
#else
  if (skeleton->priv->changed_properties_idle_source != NULL)
    g_source_destroy (skeleton->priv->changed_properties_idle_source);
skeleton->priv->changed_properties_idle_source = NULL;
#endif
  g_main_context_unref (skeleton->priv->context);
  g_mutex_clear (&skeleton->priv->lock);
  G_OBJECT_CLASS (wpa_interface_skeleton_parent_class)->finalize (object);
}

static void
wpa_interface_skeleton_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 4);
  g_mutex_lock (&skeleton->priv->lock);
  g_value_copy (&skeleton->priv->properties[prop_id - 1], value);
  g_mutex_unlock (&skeleton->priv->lock);
}

static gboolean
_wpa_interface_emit_changed (gpointer user_data)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (user_data);
  GList *l;
  GVariantBuilder builder;
  GVariantBuilder invalidated_builder;
  guint num_changes;

  g_mutex_lock (&skeleton->priv->lock);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init_static (&invalidated_builder, G_VARIANT_TYPE ("as"));
#else
  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init (&invalidated_builder, G_VARIANT_TYPE ("as"));
#endif
  for (l = skeleton->priv->changed_properties, num_changes = 0; l != NULL; l = l->next)
    {
      ChangedProperty *cp = l->data;
      GVariant *variant;
      const GValue *cur_value;

      cur_value = &skeleton->priv->properties[cp->prop_id - 1];
      if (!_g_value_equal (cur_value, &cp->orig_value))
        {
          variant = g_dbus_gvalue_to_gvariant (cur_value, G_VARIANT_TYPE (cp->info->parent_struct.signature));
          g_variant_builder_add (&builder, "{sv}", cp->info->parent_struct.name, variant);
          g_variant_unref (variant);
          num_changes++;
        }
    }
  if (num_changes > 0)
    {
      GList *connections, *ll;
      GVariant *signal_variant;
      signal_variant = g_variant_ref_sink (g_variant_new ("(sa{sv}as)", "fi.w1.wpa_supplicant1.Interface",
                                           &builder, &invalidated_builder));
      connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));
      for (ll = connections; ll != NULL; ll = ll->next)
        {
          GDBusConnection *connection = ll->data;

          g_dbus_connection_emit_signal (connection,
                                         NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)),
                                         "org.freedesktop.DBus.Properties",
                                         "PropertiesChanged",
                                         signal_variant,
                                         NULL);
        }
      g_variant_unref (signal_variant);
      g_list_free_full (connections, g_object_unref);
    }
  else
    {
      g_variant_builder_clear (&builder);
      g_variant_builder_clear (&invalidated_builder);
    }
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  skeleton->priv->changed_properties = NULL;
  skeleton->priv->changed_properties_idle_source = NULL;
  g_mutex_unlock (&skeleton->priv->lock);
  return FALSE;
}

static void
_wpa_interface_schedule_emit_changed (WPAInterfaceSkeleton *skeleton, const _ExtendedGDBusPropertyInfo *info, guint prop_id, const GValue *orig_value)
{
  ChangedProperty *cp;
  GList *l;
  cp = NULL;
  for (l = skeleton->priv->changed_properties; l != NULL; l = l->next)
    {
      ChangedProperty *i_cp = l->data;
      if (i_cp->info == info)
        {
          cp = i_cp;
          break;
        }
    }
  if (cp == NULL)
    {
      cp = g_new0 (ChangedProperty, 1);
      cp->prop_id = prop_id;
      cp->info = info;
      skeleton->priv->changed_properties = g_list_prepend (skeleton->priv->changed_properties, cp);
      g_value_init (&cp->orig_value, G_VALUE_TYPE (orig_value));
      g_value_copy (orig_value, &cp->orig_value);
    }
}

static void
wpa_interface_skeleton_notify (GObject      *object,
  GParamSpec *pspec G_GNUC_UNUSED)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (object);
  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties != NULL &&
      skeleton->priv->changed_properties_idle_source == NULL)
    {
      skeleton->priv->changed_properties_idle_source = g_idle_source_new ();
      g_source_set_priority (skeleton->priv->changed_properties_idle_source, G_PRIORITY_DEFAULT);
      g_source_set_callback (skeleton->priv->changed_properties_idle_source, _wpa_interface_emit_changed, g_object_ref (skeleton), (GDestroyNotify) g_object_unref);
      g_source_set_name (skeleton->priv->changed_properties_idle_source, "[generated] _wpa_interface_emit_changed");
      g_source_attach (skeleton->priv->changed_properties_idle_source, skeleton->priv->context);
      g_source_unref (skeleton->priv->changed_properties_idle_source);
    }
  g_mutex_unlock (&skeleton->priv->lock);
}

static void
wpa_interface_skeleton_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec)
{
  const _ExtendedGDBusPropertyInfo *info;
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 4);
  info = (const _ExtendedGDBusPropertyInfo *) _wpa_interface_property_info_pointers[prop_id - 1];
  g_mutex_lock (&skeleton->priv->lock);
  g_object_freeze_notify (object);
  if (!_g_value_equal (value, &skeleton->priv->properties[prop_id - 1]))
    {
      if (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)) != NULL &&
          info->emits_changed_signal)
        _wpa_interface_schedule_emit_changed (skeleton, info, prop_id, &skeleton->priv->properties[prop_id - 1]);
      g_value_copy (value, &skeleton->priv->properties[prop_id - 1]);
      g_object_notify_by_pspec (object, pspec);
    }
  g_mutex_unlock (&skeleton->priv->lock);
  g_object_thaw_notify (object);
}

static void
wpa_interface_skeleton_init (WPAInterfaceSkeleton *skeleton)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  skeleton->priv = wpa_interface_skeleton_get_instance_private (skeleton);
#else
  skeleton->priv = G_TYPE_INSTANCE_GET_PRIVATE (skeleton, TYPE_WPA_INTERFACE_SKELETON, WPAInterfaceSkeletonPrivate);
#endif

  g_mutex_init (&skeleton->priv->lock);
  skeleton->priv->context = g_main_context_ref_thread_default ();
  skeleton->priv->properties = g_new0 (GValue, 4);
  g_value_init (&skeleton->priv->properties[0], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[1], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[2], G_TYPE_STRING);
  g_value_init (&skeleton->priv->properties[3], G_TYPE_STRV);
}

static const gchar *
wpa_interface_skeleton_get_state (WPAInterface *object)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[0]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
wpa_interface_skeleton_get_ifname (WPAInterface *object)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[1]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *
wpa_interface_skeleton_get_driver (WPAInterface *object)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (object);
  const gchar *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_string (&(skeleton->priv->properties[2]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static const gchar *const *
wpa_interface_skeleton_get_bsss (WPAInterface *object)
{
  WPAInterfaceSkeleton *skeleton = WPA_INTERFACE_SKELETON (object);
  const gchar *const *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_boxed (&(skeleton->priv->properties[3]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static void
wpa_interface_skeleton_class_init (WPAInterfaceSkeletonClass *klass)
{
  GObjectClass *gobject_class;
  GDBusInterfaceSkeletonClass *skeleton_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = wpa_interface_skeleton_finalize;
  gobject_class->get_property = wpa_interface_skeleton_get_property;
  gobject_class->set_property = wpa_interface_skeleton_set_property;
  gobject_class->notify       = wpa_interface_skeleton_notify;


  wpa_interface_override_properties (gobject_class, 1);

  skeleton_class = G_DBUS_INTERFACE_SKELETON_CLASS (klass);
  skeleton_class->get_info = wpa_interface_skeleton_dbus_interface_get_info;
  skeleton_class->get_properties = wpa_interface_skeleton_dbus_interface_get_properties;
  skeleton_class->flush = wpa_interface_skeleton_dbus_interface_flush;
  skeleton_class->get_vtable = wpa_interface_skeleton_dbus_interface_get_vtable;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (WPAInterfaceSkeletonPrivate));
#endif
}

static void
wpa_interface_skeleton_iface_init (WPAInterfaceIface *iface)
{
  iface->bss_added = _wpa_interface_on_signal_bss_added;
  iface->bss_removed = _wpa_interface_on_signal_bss_removed;
  iface->scan_done = _wpa_interface_on_signal_scan_done;
  iface->get_state = wpa_interface_skeleton_get_state;
  iface->get_ifname = wpa_interface_skeleton_get_ifname;
  iface->get_driver = wpa_interface_skeleton_get_driver;
  iface->get_bsss = wpa_interface_skeleton_get_bsss;
}

/**
 * wpa_interface_skeleton_new:
 *
 * Creates a skeleton object for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-Interface.top_of_page">fi.w1.wpa_supplicant1.Interface</link>.
 *
 * Returns: (transfer full) (type WPAInterfaceSkeleton): The skeleton object.
 */
WPAInterface *
wpa_interface_skeleton_new (void)
{
  return WPA_INTERFACE (g_object_new (TYPE_WPA_INTERFACE_SKELETON, NULL));
}

/* ------------------------------------------------------------------------
 * Code for interface fi.w1.wpa_supplicant1.BSS
 * ------------------------------------------------------------------------
 */

/**
 * SECTION:WPABSS
 * @title: WPABSS
 * @short_description: Generated C code for the fi.w1.wpa_supplicant1.BSS D-Bus interface
 *
 * This section contains code for working with the <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-BSS.top_of_page">fi.w1.wpa_supplicant1.BSS</link> D-Bus interface in C.
 */

/* ---- Introspection data for fi.w1.wpa_supplicant1.BSS ---- */

static const _ExtendedGDBusPropertyInfo _wpa_bss_property_info_ssid =
{
  {
    -1,
    (gchar *) "SSID",
    (gchar *) "ay",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "ssid",
  TRUE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _wpa_bss_property_info_bssid =
{
  {
    -1,
    (gchar *) "BSSID",
    (gchar *) "ay",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "bssid",
  TRUE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _wpa_bss_property_info_signal =
{
  {
    -1,
    (gchar *) "Signal",
    (gchar *) "n",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "signal",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _wpa_bss_property_info_frequency =
{
  {
    -1,
    (gchar *) "Frequency",
    (gchar *) "q",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "frequency",
  FALSE,
  TRUE
};

static const _ExtendedGDBusPropertyInfo _wpa_bss_property_info_age =
{
  {
    -1,
    (gchar *) "Age",
    (gchar *) "u",
    G_DBUS_PROPERTY_INFO_FLAGS_READABLE,
    NULL
  },
  "age",
  FALSE,
  TRUE
};

static const GDBusPropertyInfo * const _wpa_bss_property_info_pointers[] =
{
  &_wpa_bss_property_info_ssid.parent_struct,
  &_wpa_bss_property_info_bssid.parent_struct,
  &_wpa_bss_property_info_signal.parent_struct,
  &_wpa_bss_property_info_frequency.parent_struct,
  &_wpa_bss_property_info_age.parent_struct,
  NULL
};

static const _ExtendedGDBusInterfaceInfo _wpa_bss_interface_info =
{
  {
    -1,
    (gchar *) "fi.w1.wpa_supplicant1.BSS",
    NULL,
    NULL,
    (GDBusPropertyInfo **) &_wpa_bss_property_info_pointers,
    NULL
  },
  "wpa-bss",
};


/**
 * wpa_bss_interface_info:
 *
 * Gets a machine-readable description of the <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-BSS.top_of_page">fi.w1.wpa_supplicant1.BSS</link> D-Bus interface.
 *
 * Returns: (transfer none): A #GDBusInterfaceInfo. Do not free.
 */
GDBusInterfaceInfo *
wpa_bss_interface_info (void)
{
  return (GDBusInterfaceInfo *) &_wpa_bss_interface_info.parent_struct;
}

/**
 * wpa_bss_override_properties:
 * @klass: The class structure for a #GObject derived class.
 * @property_id_begin: The property id to assign to the first overridden property.
 *
 * Overrides all #GObject properties in the #WPABSS interface for a concrete class.
 * The properties are overridden in the order they are defined.
 *
 * Returns: The last property id.
 */
guint
wpa_bss_override_properties (GObjectClass *klass, guint property_id_begin)
{
  g_object_class_override_property (klass, property_id_begin++, "ssid");
  g_object_class_override_property (klass, property_id_begin++, "bssid");
  g_object_class_override_property (klass, property_id_begin++, "signal");
  g_object_class_override_property (klass, property_id_begin++, "frequency");
  g_object_class_override_property (klass, property_id_begin++, "age");
  return property_id_begin - 1;
}



/**
 * WPABSS:
 *
 * Abstract interface type for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-BSS.top_of_page">fi.w1.wpa_supplicant1.BSS</link>.
 */

/**
 * WPABSSIface:
 * @parent_iface: The parent interface.
 * @get_age: Getter for the #WPABSS:age property.
 * @get_bssid: Getter for the #WPABSS:bssid property.
 * @get_frequency: Getter for the #WPABSS:frequency property.
 * @get_signal: Getter for the #WPABSS:signal property.
 * @get_ssid: Getter for the #WPABSS:ssid property.
 *
 * Virtual table for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-BSS.top_of_page">fi.w1.wpa_supplicant1.BSS</link>.
 */

typedef WPABSSIface WPABSSInterface;
G_DEFINE_INTERFACE (WPABSS, wpa_bss, G_TYPE_OBJECT)

static void
wpa_bss_default_init (WPABSSIface *iface)
{
  /* GObject properties for D-Bus properties: */
  /**
   * WPABSS:ssid:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.SSID">"SSID"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_variant ("ssid", "SSID", "SSID", G_VARIANT_TYPE ("ay"), NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * WPABSS:bssid:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.BSSID">"BSSID"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_variant ("bssid", "BSSID", "BSSID", G_VARIANT_TYPE ("ay"), NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * WPABSS:signal:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.Signal">"Signal"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_int ("signal", "Signal", "Signal", G_MININT16, G_MAXINT16, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * WPABSS:frequency:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.Frequency">"Frequency"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_uint ("frequency", "Frequency", "Frequency", 0, G_MAXUINT16, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
  /**
   * WPABSS:age:
   *
   * Represents the D-Bus property <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.Age">"Age"</link>.
   *
   * Since the D-Bus property for this #GObject property is readable but not writable, it is meaningful to read from it on both the client- and service-side. It is only meaningful, however, to write to it on the service-side.
   */
  g_object_interface_install_property (iface,
    g_param_spec_uint ("age", "Age", "Age", 0, G_MAXUINT32, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
}

/**
 * wpa_bss_get_ssid: (skip)
 * @object: A #WPABSS.
 *
 * Gets the value of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.SSID">"SSID"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use wpa_bss_dup_ssid() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
GVariant *
wpa_bss_get_ssid (WPABSS *object)
{
  g_return_val_if_fail (IS_WPA_BSS (object), NULL);

  return WPA_BSS_GET_IFACE (object)->get_ssid (object);
}

/**
 * wpa_bss_dup_ssid: (skip)
 * @object: A #WPABSS.
 *
 * Gets a copy of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.SSID">"SSID"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_variant_unref().
 */
GVariant *
wpa_bss_dup_ssid (WPABSS *object)
{
  GVariant *value;
  g_object_get (G_OBJECT (object), "ssid", &value, NULL);
  return value;
}

/**
 * wpa_bss_set_ssid: (skip)
 * @object: A #WPABSS.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.SSID">"SSID"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
wpa_bss_set_ssid (WPABSS *object, GVariant *value)
{
  g_object_set (G_OBJECT (object), "ssid", value, NULL);
}

/**
 * wpa_bss_get_bssid: (skip)
 * @object: A #WPABSS.
 *
 * Gets the value of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.BSSID">"BSSID"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * The returned value is only valid until the property changes so on the client-side it is only safe to use this function on the thread where @object was constructed. Use wpa_bss_dup_bssid() if on another thread.
 *
 * Returns: (transfer none) (nullable): The property value or %NULL if the property is not set. Do not free the returned value, it belongs to @object.
 */
GVariant *
wpa_bss_get_bssid (WPABSS *object)
{
  g_return_val_if_fail (IS_WPA_BSS (object), NULL);

  return WPA_BSS_GET_IFACE (object)->get_bssid (object);
}

/**
 * wpa_bss_dup_bssid: (skip)
 * @object: A #WPABSS.
 *
 * Gets a copy of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.BSSID">"BSSID"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: (transfer full) (nullable): The property value or %NULL if the property is not set. The returned value should be freed with g_variant_unref().
 */
GVariant *
wpa_bss_dup_bssid (WPABSS *object)
{
  GVariant *value;
  g_object_get (G_OBJECT (object), "bssid", &value, NULL);
  return value;
}

/**
 * wpa_bss_set_bssid: (skip)
 * @object: A #WPABSS.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.BSSID">"BSSID"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
wpa_bss_set_bssid (WPABSS *object, GVariant *value)
{
  g_object_set (G_OBJECT (object), "bssid", value, NULL);
}

/**
 * wpa_bss_get_signal: (skip)
 * @object: A #WPABSS.
 *
 * Gets the value of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.Signal">"Signal"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
gint16 
wpa_bss_get_signal (WPABSS *object)
{
  g_return_val_if_fail (IS_WPA_BSS (object), 0);

  return WPA_BSS_GET_IFACE (object)->get_signal (object);
}

/**
 * wpa_bss_set_signal: (skip)
 * @object: A #WPABSS.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.Signal">"Signal"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
wpa_bss_set_signal (WPABSS *object, gint16 value)
{
  g_object_set (G_OBJECT (object), "signal", value, NULL);
}

/**
 * wpa_bss_get_frequency: (skip)
 * @object: A #WPABSS.
 *
 * Gets the value of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.Frequency">"Frequency"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
guint16 
wpa_bss_get_frequency (WPABSS *object)
{
  g_return_val_if_fail (IS_WPA_BSS (object), 0);

  return WPA_BSS_GET_IFACE (object)->get_frequency (object);
}

/**
 * wpa_bss_set_frequency: (skip)
 * @object: A #WPABSS.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.Frequency">"Frequency"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
wpa_bss_set_frequency (WPABSS *object, guint16 value)
{
  g_object_set (G_OBJECT (object), "frequency", value, NULL);
}

/**
 * wpa_bss_get_age: (skip)
 * @object: A #WPABSS.
 *
 * Gets the value of the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.Age">"Age"</link> D-Bus property.
 *
 * Since this D-Bus property is readable, it is meaningful to use this function on both the client- and service-side.
 *
 * Returns: The property value.
 */
guint 
wpa_bss_get_age (WPABSS *object)
{
  g_return_val_if_fail (IS_WPA_BSS (object), 0);

  return WPA_BSS_GET_IFACE (object)->get_age (object);
}

/**
 * wpa_bss_set_age: (skip)
 * @object: A #WPABSS.
 * @value: The value to set.
 *
 * Sets the <link linkend="gdbus-property-fi-w1-wpa_supplicant1-BSS.Age">"Age"</link> D-Bus property to @value.
 *
 * Since this D-Bus property is not writable, it is only meaningful to use this function on the service-side.
 */
void
wpa_bss_set_age (WPABSS *object, guint value)
{
  g_object_set (G_OBJECT (object), "age", value, NULL);
}

/* ------------------------------------------------------------------------ */

/**
 * WPABSSProxy:
 *
 * The #WPABSSProxy structure contains only private data and should only be accessed using the provided API.
 */

/**
 * WPABSSProxyClass:
 * @parent_class: The parent class.
 *
 * Class structure for #WPABSSProxy.
 */

struct _WPABSSProxyPrivate
{
  GData *qdata;
};

static void wpa_bss_proxy_iface_init (WPABSSIface *iface);

#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (WPABSSProxy, wpa_bss_proxy, G_TYPE_DBUS_PROXY,
                         G_ADD_PRIVATE (WPABSSProxy)
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_BSS, wpa_bss_proxy_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (WPABSSProxy, wpa_bss_proxy, G_TYPE_DBUS_PROXY,
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_BSS, wpa_bss_proxy_iface_init))

#endif
static void
wpa_bss_proxy_finalize (GObject *object)
{
  WPABSSProxy *proxy = WPA_BSS_PROXY (object);
  g_datalist_clear (&proxy->priv->qdata);
  G_OBJECT_CLASS (wpa_bss_proxy_parent_class)->finalize (object);
}

static void
wpa_bss_proxy_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 5);
  info = (const _ExtendedGDBusPropertyInfo *) _wpa_bss_property_info_pointers[prop_id - 1];
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (object), info->parent_struct.name);
  if (info->use_gvariant)
    {
      g_value_set_variant (value, variant);
    }
  else
    {
      if (variant != NULL)
        g_dbus_gvariant_to_gvalue (variant, value);
    }
  if (variant != NULL)
    g_variant_unref (variant);
}

static void
wpa_bss_proxy_set_property_cb (GDBusProxy *proxy,
  GAsyncResult *res,
  gpointer      user_data)
{
  const _ExtendedGDBusPropertyInfo *info = user_data;
  GError *error;
  GVariant *_ret;
  error = NULL;
  _ret = g_dbus_proxy_call_finish (proxy, res, &error);
  if (!_ret)
    {
      g_warning ("Error setting property '%s' on interface fi.w1.wpa_supplicant1.BSS: %s (%s, %d)",
                 info->parent_struct.name, 
                 error->message, g_quark_to_string (error->domain), error->code);
      g_error_free (error);
    }
  else
    {
      g_variant_unref (_ret);
    }
}

static void
wpa_bss_proxy_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  const _ExtendedGDBusPropertyInfo *info;
  GVariant *variant;
  g_assert (prop_id != 0 && prop_id - 1 < 5);
  info = (const _ExtendedGDBusPropertyInfo *) _wpa_bss_property_info_pointers[prop_id - 1];
  variant = g_dbus_gvalue_to_gvariant (value, G_VARIANT_TYPE (info->parent_struct.signature));
  g_dbus_proxy_call (G_DBUS_PROXY (object),
    "org.freedesktop.DBus.Properties.Set",
    g_variant_new ("(ssv)", "fi.w1.wpa_supplicant1.BSS", info->parent_struct.name, variant),
    G_DBUS_CALL_FLAGS_NONE,
    -1,
    NULL, (GAsyncReadyCallback) wpa_bss_proxy_set_property_cb, (GDBusPropertyInfo *) &info->parent_struct);
  g_variant_unref (variant);
}

static void
wpa_bss_proxy_g_signal (GDBusProxy *proxy,
  const gchar *sender_name G_GNUC_UNUSED,
  const gchar *signal_name,
  GVariant *parameters)
{
  _ExtendedGDBusSignalInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  gsize n;
  guint signal_id;
  info = (_ExtendedGDBusSignalInfo *) g_dbus_interface_info_lookup_signal ((GDBusInterfaceInfo *) &_wpa_bss_interface_info.parent_struct, signal_name);
  if (info == NULL)
    return;
  num_params = g_variant_n_children (parameters);
  paramv = g_new0 (GValue, num_params + 1);
  g_value_init (&paramv[0], TYPE_WPA_BSS);
  g_value_set_object (&paramv[0], proxy);
  g_variant_iter_init (&iter, parameters);
  n = 1;
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.args[n - 1];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_WPA_BSS);
  g_signal_emitv (paramv, signal_id, 0, NULL);
  for (n = 0; n < num_params + 1; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static void
wpa_bss_proxy_g_properties_changed (GDBusProxy *_proxy,
  GVariant *changed_properties,
  const gchar *const *invalidated_properties)
{
  WPABSSProxy *proxy = WPA_BSS_PROXY (_proxy);
  guint n;
  const gchar *key;
  GVariantIter *iter;
  _ExtendedGDBusPropertyInfo *info;
  g_variant_get (changed_properties, "a{sv}", &iter);
  while (g_variant_iter_next (iter, "{&sv}", &key, NULL))
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_bss_interface_info.parent_struct, key);
      g_datalist_remove_data (&proxy->priv->qdata, key);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
  g_variant_iter_free (iter);
  for (n = 0; invalidated_properties[n] != NULL; n++)
    {
      info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_bss_interface_info.parent_struct, invalidated_properties[n]);
      g_datalist_remove_data (&proxy->priv->qdata, invalidated_properties[n]);
      if (info != NULL)
        g_object_notify (G_OBJECT (proxy), info->hyphen_name);
    }
}

static GVariant *
wpa_bss_proxy_get_ssid (WPABSS *object)
{
  WPABSSProxy *proxy = WPA_BSS_PROXY (object);
  GVariant *variant;
  GVariant *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "SSID");
  value = variant;
  if (variant != NULL)
    g_variant_unref (variant);
  return value;
}

static GVariant *
wpa_bss_proxy_get_bssid (WPABSS *object)
{
  WPABSSProxy *proxy = WPA_BSS_PROXY (object);
  GVariant *variant;
  GVariant *value = NULL;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "BSSID");
  value = variant;
  if (variant != NULL)
    g_variant_unref (variant);
  return value;
}

static gint16 
wpa_bss_proxy_get_signal (WPABSS *object)
{
  WPABSSProxy *proxy = WPA_BSS_PROXY (object);
  GVariant *variant;
  gint16 value = 0;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Signal");
  if (variant != NULL)
    {
      value = g_variant_get_int16 (variant);
      g_variant_unref (variant);
    }
  return value;
}

static guint16 
wpa_bss_proxy_get_frequency (WPABSS *object)
{
  WPABSSProxy *proxy = WPA_BSS_PROXY (object);
  GVariant *variant;
  guint16 value = 0;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Frequency");
  if (variant != NULL)
    {
      value = g_variant_get_uint16 (variant);
      g_variant_unref (variant);
    }
  return value;
}

static guint 
wpa_bss_proxy_get_age (WPABSS *object)
{
  WPABSSProxy *proxy = WPA_BSS_PROXY (object);
  GVariant *variant;
  guint value = 0;
  variant = g_dbus_proxy_get_cached_property (G_DBUS_PROXY (proxy), "Age");
  if (variant != NULL)
    {
      value = g_variant_get_uint32 (variant);
      g_variant_unref (variant);
    }
  return value;
}

static void
wpa_bss_proxy_init (WPABSSProxy *proxy)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  proxy->priv = wpa_bss_proxy_get_instance_private (proxy);
#else
  proxy->priv = G_TYPE_INSTANCE_GET_PRIVATE (proxy, TYPE_WPA_BSS_PROXY, WPABSSProxyPrivate);
#endif

  g_dbus_proxy_set_interface_info (G_DBUS_PROXY (proxy), wpa_bss_interface_info ());
}

static void
wpa_bss_proxy_class_init (WPABSSProxyClass *klass)
{
  GObjectClass *gobject_class;
  GDBusProxyClass *proxy_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize     = wpa_bss_proxy_finalize;
  gobject_class->get_property = wpa_bss_proxy_get_property;
  gobject_class->set_property = wpa_bss_proxy_set_property;

  proxy_class = G_DBUS_PROXY_CLASS (klass);
  proxy_class->g_signal = wpa_bss_proxy_g_signal;
  proxy_class->g_properties_changed = wpa_bss_proxy_g_properties_changed;

  wpa_bss_override_properties (gobject_class, 1);

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (WPABSSProxyPrivate));
#endif
}

static void
wpa_bss_proxy_iface_init (WPABSSIface *iface)
{
  iface->get_ssid = wpa_bss_proxy_get_ssid;
  iface->get_bssid = wpa_bss_proxy_get_bssid;
  iface->get_signal = wpa_bss_proxy_get_signal;
  iface->get_frequency = wpa_bss_proxy_get_frequency;
  iface->get_age = wpa_bss_proxy_get_age;
}

/**
 * wpa_bss_proxy_new:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Asynchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-BSS.top_of_page">fi.w1.wpa_supplicant1.BSS</link>. See g_dbus_proxy_new() for more details.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call wpa_bss_proxy_new_finish() to get the result of the operation.
 *
 * See wpa_bss_proxy_new_sync() for the synchronous, blocking version of this constructor.
 */
void
wpa_bss_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_WPA_BSS_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1.BSS", NULL);
}

/**
 * wpa_bss_proxy_new_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to wpa_bss_proxy_new().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with wpa_bss_proxy_new().
 *
 * Returns: (transfer full) (type WPABSSProxy): The constructed proxy object or %NULL if @error is set.
 */
WPABSS *
wpa_bss_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return WPA_BSS (ret);
  else
    return NULL;
}

/**
 * wpa_bss_proxy_new_sync:
 * @connection: A #GDBusConnection.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: (nullable): A bus name (well-known or unique) or %NULL if @connection is not a message bus connection.
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Synchronously creates a proxy for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-BSS.top_of_page">fi.w1.wpa_supplicant1.BSS</link>. See g_dbus_proxy_new_sync() for more details.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See wpa_bss_proxy_new() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type WPABSSProxy): The constructed proxy object or %NULL if @error is set.
 */
WPABSS *
wpa_bss_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_WPA_BSS_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-connection", connection, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1.BSS", NULL);
  if (ret != NULL)
    return WPA_BSS (ret);
  else
    return NULL;
}


/**
 * wpa_bss_proxy_new_for_bus:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @callback: A #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: User data to pass to @callback.
 *
 * Like wpa_bss_proxy_new() but takes a #GBusType instead of a #GDBusConnection.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from (see g_main_context_push_thread_default()).
 * You can then call wpa_bss_proxy_new_for_bus_finish() to get the result of the operation.
 *
 * See wpa_bss_proxy_new_for_bus_sync() for the synchronous, blocking version of this constructor.
 */
void
wpa_bss_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data)
{
  g_async_initable_new_async (TYPE_WPA_BSS_PROXY, G_PRIORITY_DEFAULT, cancellable, callback, user_data, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1.BSS", NULL);
}

/**
 * wpa_bss_proxy_new_for_bus_finish:
 * @res: The #GAsyncResult obtained from the #GAsyncReadyCallback passed to wpa_bss_proxy_new_for_bus().
 * @error: Return location for error or %NULL
 *
 * Finishes an operation started with wpa_bss_proxy_new_for_bus().
 *
 * Returns: (transfer full) (type WPABSSProxy): The constructed proxy object or %NULL if @error is set.
 */
WPABSS *
wpa_bss_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error)
{
  GObject *ret;
  GObject *source_object;
  source_object = g_async_result_get_source_object (res);
  ret = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
  g_object_unref (source_object);
  if (ret != NULL)
    return WPA_BSS (ret);
  else
    return NULL;
}

/**
 * wpa_bss_proxy_new_for_bus_sync:
 * @bus_type: A #GBusType.
 * @flags: Flags from the #GDBusProxyFlags enumeration.
 * @name: A bus name (well-known or unique).
 * @object_path: An object path.
 * @cancellable: (nullable): A #GCancellable or %NULL.
 * @error: Return location for error or %NULL
 *
 * Like wpa_bss_proxy_new_sync() but takes a #GBusType instead of a #GDBusConnection.
 *
 * The calling thread is blocked until a reply is received.
 *
 * See wpa_bss_proxy_new_for_bus() for the asynchronous version of this constructor.
 *
 * Returns: (transfer full) (type WPABSSProxy): The constructed proxy object or %NULL if @error is set.
 */
WPABSS *
wpa_bss_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error)
{
  GInitable *ret;
  ret = g_initable_new (TYPE_WPA_BSS_PROXY, cancellable, error, "g-flags", flags, "g-name", name, "g-bus-type", bus_type, "g-object-path", object_path, "g-interface-name", "fi.w1.wpa_supplicant1.BSS", NULL);
  if (ret != NULL)
    return WPA_BSS (ret);
  else
    return NULL;
}


/* ------------------------------------------------------------------------ */

/**
 * WPABSSSkeleton:
 *
 * The #WPABSSSkeleton structure contains only private data and should only be accessed using the provided API.
 */

/**
 * WPABSSSkeletonClass:
 * @parent_class: The parent class.
 *
 * Class structure for #WPABSSSkeleton.
 */

struct _WPABSSSkeletonPrivate
{
  GValue *properties;
  GList *changed_properties;
  GSource *changed_properties_idle_source;
  GMainContext *context;
  GMutex lock;
};

static void
_wpa_bss_skeleton_handle_method_call (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name,
  const gchar *method_name,
  GVariant *parameters,
  GDBusMethodInvocation *invocation,
  gpointer user_data)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (user_data);
  _ExtendedGDBusMethodInfo *info;
  GVariantIter iter;
  GVariant *child;
  GValue *paramv;
  gsize num_params;
  guint num_extra;
  gsize n;
  guint signal_id;
  GValue return_value = G_VALUE_INIT;
  info = (_ExtendedGDBusMethodInfo *) g_dbus_method_invocation_get_method_info (invocation);
  g_assert (info != NULL);
  num_params = g_variant_n_children (parameters);
  num_extra = info->pass_fdlist ? 3 : 2;  paramv = g_new0 (GValue, num_params + num_extra);
  n = 0;
  g_value_init (&paramv[n], TYPE_WPA_BSS);
  g_value_set_object (&paramv[n++], skeleton);
  g_value_init (&paramv[n], G_TYPE_DBUS_METHOD_INVOCATION);
  g_value_set_object (&paramv[n++], invocation);
  if (info->pass_fdlist)
    {
#ifdef G_OS_UNIX
      g_value_init (&paramv[n], G_TYPE_UNIX_FD_LIST);
      g_value_set_object (&paramv[n++], g_dbus_message_get_unix_fd_list (g_dbus_method_invocation_get_message (invocation)));
#else
      g_assert_not_reached ();
#endif
    }
  g_variant_iter_init (&iter, parameters);
  while ((child = g_variant_iter_next_value (&iter)) != NULL)
    {
      _ExtendedGDBusArgInfo *arg_info = (_ExtendedGDBusArgInfo *) info->parent_struct.in_args[n - num_extra];
      if (arg_info->use_gvariant)
        {
          g_value_init (&paramv[n], G_TYPE_VARIANT);
          g_value_set_variant (&paramv[n], child);
          n++;
        }
      else
        g_dbus_gvariant_to_gvalue (child, &paramv[n++]);
      g_variant_unref (child);
    }
  signal_id = g_signal_lookup (info->signal_name, TYPE_WPA_BSS);
  g_value_init (&return_value, G_TYPE_BOOLEAN);
  g_signal_emitv (paramv, signal_id, 0, &return_value);
  if (!g_value_get_boolean (&return_value))
    g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD, "Method %s is not implemented on interface %s", method_name, interface_name);
  g_value_unset (&return_value);
  for (n = 0; n < num_params + num_extra; n++)
    g_value_unset (&paramv[n]);
  g_free (paramv);
}

static GVariant *
_wpa_bss_skeleton_handle_get_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GError **error,
  gpointer user_data)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  GVariant *ret;
  ret = NULL;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_bss_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      g_value_init (&value, pspec->value_type);
      g_object_get_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      ret = g_dbus_gvalue_to_gvariant (&value, G_VARIANT_TYPE (info->parent_struct.signature));
      g_value_unset (&value);
    }
  return ret;
}

static gboolean
_wpa_bss_skeleton_handle_set_property (
  GDBusConnection *connection G_GNUC_UNUSED,
  const gchar *sender G_GNUC_UNUSED,
  const gchar *object_path G_GNUC_UNUSED,
  const gchar *interface_name G_GNUC_UNUSED,
  const gchar *property_name,
  GVariant *variant,
  GError **error,
  gpointer user_data)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (user_data);
  GValue value = G_VALUE_INIT;
  GParamSpec *pspec;
  _ExtendedGDBusPropertyInfo *info;
  gboolean ret;
  ret = FALSE;
  info = (_ExtendedGDBusPropertyInfo *) g_dbus_interface_info_lookup_property ((GDBusInterfaceInfo *) &_wpa_bss_interface_info.parent_struct, property_name);
  g_assert (info != NULL);
  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (skeleton), info->hyphen_name);
  if (pspec == NULL)
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "No property with name %s", property_name);
    }
  else
    {
      if (info->use_gvariant)
        g_value_set_variant (&value, variant);
      else
        g_dbus_gvariant_to_gvalue (variant, &value);
      g_object_set_property (G_OBJECT (skeleton), info->hyphen_name, &value);
      g_value_unset (&value);
      ret = TRUE;
    }
  return ret;
}

static const GDBusInterfaceVTable _wpa_bss_skeleton_vtable =
{
  _wpa_bss_skeleton_handle_method_call,
  _wpa_bss_skeleton_handle_get_property,
  _wpa_bss_skeleton_handle_set_property,
  {NULL}
};

static GDBusInterfaceInfo *
wpa_bss_skeleton_dbus_interface_get_info (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return wpa_bss_interface_info ();
}

static GDBusInterfaceVTable *
wpa_bss_skeleton_dbus_interface_get_vtable (GDBusInterfaceSkeleton *skeleton G_GNUC_UNUSED)
{
  return (GDBusInterfaceVTable *) &_wpa_bss_skeleton_vtable;
}

static GVariant *
wpa_bss_skeleton_dbus_interface_get_properties (GDBusInterfaceSkeleton *_skeleton)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (_skeleton);

  GVariantBuilder builder;
  guint n;
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
#else
  g_variant_builder_init(&builder, G_VARIANT_TYPE ("a{sv}"));
#endif
  if (_wpa_bss_interface_info.parent_struct.properties == NULL)
    goto out;
  for (n = 0; _wpa_bss_interface_info.parent_struct.properties[n] != NULL; n++)
    {
      GDBusPropertyInfo *info = _wpa_bss_interface_info.parent_struct.properties[n];
      if (info->flags & G_DBUS_PROPERTY_INFO_FLAGS_READABLE)
        {
          GVariant *value;
          value = _wpa_bss_skeleton_handle_get_property (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)), NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)), "fi.w1.wpa_supplicant1.BSS", info->name, NULL, skeleton);
          if (value != NULL)
            {
              g_variant_take_ref (value);
              g_variant_builder_add (&builder, "{sv}", info->name, value);
              g_variant_unref (value);
            }
        }
    }
out:
  return g_variant_builder_end (&builder);
}

static gboolean _wpa_bss_emit_changed (gpointer user_data);

static void
wpa_bss_skeleton_dbus_interface_flush (GDBusInterfaceSkeleton *_skeleton)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (_skeleton);
  gboolean emit_changed = FALSE;

  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties_idle_source != NULL)
    {
      g_source_destroy (skeleton->priv->changed_properties_idle_source);
      skeleton->priv->changed_properties_idle_source = NULL;
      emit_changed = TRUE;
    }
  g_mutex_unlock (&skeleton->priv->lock);

  if (emit_changed)
    _wpa_bss_emit_changed (skeleton);
}

static void wpa_bss_skeleton_iface_init (WPABSSIface *iface);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
G_DEFINE_TYPE_WITH_CODE (WPABSSSkeleton, wpa_bss_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_ADD_PRIVATE (WPABSSSkeleton)
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_BSS, wpa_bss_skeleton_iface_init))

#else
G_DEFINE_TYPE_WITH_CODE (WPABSSSkeleton, wpa_bss_skeleton, G_TYPE_DBUS_INTERFACE_SKELETON,
                         G_IMPLEMENT_INTERFACE (TYPE_WPA_BSS, wpa_bss_skeleton_iface_init))

#endif
static void
wpa_bss_skeleton_finalize (GObject *object)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (object);
  guint n;
  for (n = 0; n < 5; n++)
    g_value_unset (&skeleton->priv->properties[n]);
  g_free (skeleton->priv->properties);
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  /* coverity[missing_lock : SUPPRESS] */
  g_clear_pointer (&skeleton->priv->changed_properties_idle_source, g_source_destroy);
#else
  if (skeleton->priv->changed_properties_idle_source != NULL)
    g_source_destroy (skeleton->priv->changed_properties_idle_source);
skeleton->priv->changed_properties_idle_source = NULL;
#endif
  g_main_context_unref (skeleton->priv->context);
  g_mutex_clear (&skeleton->priv->lock);
  G_OBJECT_CLASS (wpa_bss_skeleton_parent_class)->finalize (object);
}

static void
wpa_bss_skeleton_get_property (GObject      *object,
  guint         prop_id,
  GValue       *value,
  GParamSpec   *pspec G_GNUC_UNUSED)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 5);
  g_mutex_lock (&skeleton->priv->lock);
  g_value_copy (&skeleton->priv->properties[prop_id - 1], value);
  g_mutex_unlock (&skeleton->priv->lock);
}

static gboolean
_wpa_bss_emit_changed (gpointer user_data)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (user_data);
  GList *l;
  GVariantBuilder builder;
  GVariantBuilder invalidated_builder;
  guint num_changes;

  g_mutex_lock (&skeleton->priv->lock);
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_84
  g_variant_builder_init_static (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init_static (&invalidated_builder, G_VARIANT_TYPE ("as"));
#else
  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
  g_variant_builder_init (&invalidated_builder, G_VARIANT_TYPE ("as"));
#endif
  for (l = skeleton->priv->changed_properties, num_changes = 0; l != NULL; l = l->next)
    {
      ChangedProperty *cp = l->data;
      GVariant *variant;
      const GValue *cur_value;

      cur_value = &skeleton->priv->properties[cp->prop_id - 1];
      if (!_g_value_equal (cur_value, &cp->orig_value))
        {
          variant = g_dbus_gvalue_to_gvariant (cur_value, G_VARIANT_TYPE (cp->info->parent_struct.signature));
          g_variant_builder_add (&builder, "{sv}", cp->info->parent_struct.name, variant);
          g_variant_unref (variant);
          num_changes++;
        }
    }
  if (num_changes > 0)
    {
      GList *connections, *ll;
      GVariant *signal_variant;
      signal_variant = g_variant_ref_sink (g_variant_new ("(sa{sv}as)", "fi.w1.wpa_supplicant1.BSS",
                                           &builder, &invalidated_builder));
      connections = g_dbus_interface_skeleton_get_connections (G_DBUS_INTERFACE_SKELETON (skeleton));
      for (ll = connections; ll != NULL; ll = ll->next)
        {
          GDBusConnection *connection = ll->data;

          g_dbus_connection_emit_signal (connection,
                                         NULL, g_dbus_interface_skeleton_get_object_path (G_DBUS_INTERFACE_SKELETON (skeleton)),
                                         "org.freedesktop.DBus.Properties",
                                         "PropertiesChanged",
                                         signal_variant,
                                         NULL);
        }
      g_variant_unref (signal_variant);
      g_list_free_full (connections, g_object_unref);
    }
  else
    {
      g_variant_builder_clear (&builder);
      g_variant_builder_clear (&invalidated_builder);
    }
  g_list_free_full (skeleton->priv->changed_properties, (GDestroyNotify) _changed_property_free);
  skeleton->priv->changed_properties = NULL;
  skeleton->priv->changed_properties_idle_source = NULL;
  g_mutex_unlock (&skeleton->priv->lock);
  return FALSE;
}

static void
_wpa_bss_schedule_emit_changed (WPABSSSkeleton *skeleton, const _ExtendedGDBusPropertyInfo *info, guint prop_id, const GValue *orig_value)
{
  ChangedProperty *cp;
  GList *l;
  cp = NULL;
  for (l = skeleton->priv->changed_properties; l != NULL; l = l->next)
    {
      ChangedProperty *i_cp = l->data;
      if (i_cp->info == info)
        {
          cp = i_cp;
          break;
        }
    }
  if (cp == NULL)
    {
      cp = g_new0 (ChangedProperty, 1);
      cp->prop_id = prop_id;
      cp->info = info;
      skeleton->priv->changed_properties = g_list_prepend (skeleton->priv->changed_properties, cp);
      g_value_init (&cp->orig_value, G_VALUE_TYPE (orig_value));
      g_value_copy (orig_value, &cp->orig_value);
    }
}

static void
wpa_bss_skeleton_notify (GObject      *object,
  GParamSpec *pspec G_GNUC_UNUSED)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (object);
  g_mutex_lock (&skeleton->priv->lock);
  if (skeleton->priv->changed_properties != NULL &&
      skeleton->priv->changed_properties_idle_source == NULL)
    {
      skeleton->priv->changed_properties_idle_source = g_idle_source_new ();
      g_source_set_priority (skeleton->priv->changed_properties_idle_source, G_PRIORITY_DEFAULT);
      g_source_set_callback (skeleton->priv->changed_properties_idle_source, _wpa_bss_emit_changed, g_object_ref (skeleton), (GDestroyNotify) g_object_unref);
      g_source_set_name (skeleton->priv->changed_properties_idle_source, "[generated] _wpa_bss_emit_changed");
      g_source_attach (skeleton->priv->changed_properties_idle_source, skeleton->priv->context);
      g_source_unref (skeleton->priv->changed_properties_idle_source);
    }
  g_mutex_unlock (&skeleton->priv->lock);
}

static void
wpa_bss_skeleton_set_property (GObject      *object,
  guint         prop_id,
  const GValue *value,
  GParamSpec   *pspec)
{
  const _ExtendedGDBusPropertyInfo *info;
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (object);
  g_assert (prop_id != 0 && prop_id - 1 < 5);
  info = (const _ExtendedGDBusPropertyInfo *) _wpa_bss_property_info_pointers[prop_id - 1];
  g_mutex_lock (&skeleton->priv->lock);
  g_object_freeze_notify (object);
  if (!_g_value_equal (value, &skeleton->priv->properties[prop_id - 1]))
    {
      if (g_dbus_interface_skeleton_get_connection (G_DBUS_INTERFACE_SKELETON (skeleton)) != NULL &&
          info->emits_changed_signal)
        _wpa_bss_schedule_emit_changed (skeleton, info, prop_id, &skeleton->priv->properties[prop_id - 1]);
      g_value_copy (value, &skeleton->priv->properties[prop_id - 1]);
      g_object_notify_by_pspec (object, pspec);
    }
  g_mutex_unlock (&skeleton->priv->lock);
  g_object_thaw_notify (object);
}

static void
wpa_bss_skeleton_init (WPABSSSkeleton *skeleton)
{
#if GLIB_VERSION_MAX_ALLOWED >= GLIB_VERSION_2_38
  skeleton->priv = wpa_bss_skeleton_get_instance_private (skeleton);
#else
  skeleton->priv = G_TYPE_INSTANCE_GET_PRIVATE (skeleton, TYPE_WPA_BSS_SKELETON, WPABSSSkeletonPrivate);
#endif

  g_mutex_init (&skeleton->priv->lock);
  skeleton->priv->context = g_main_context_ref_thread_default ();
  skeleton->priv->properties = g_new0 (GValue, 5);
  g_value_init (&skeleton->priv->properties[0], G_TYPE_VARIANT);
  g_value_init (&skeleton->priv->properties[1], G_TYPE_VARIANT);
  g_value_init (&skeleton->priv->properties[2], G_TYPE_INT);
  g_value_init (&skeleton->priv->properties[3], G_TYPE_UINT);
  g_value_init (&skeleton->priv->properties[4], G_TYPE_UINT);
}

static GVariant *
wpa_bss_skeleton_get_ssid (WPABSS *object)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (object);
  GVariant *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_variant (&(skeleton->priv->properties[0]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static GVariant *
wpa_bss_skeleton_get_bssid (WPABSS *object)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (object);
  GVariant *value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_variant (&(skeleton->priv->properties[1]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static gint16 
wpa_bss_skeleton_get_signal (WPABSS *object)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (object);
  gint value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_int (&(skeleton->priv->properties[2]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static guint16 
wpa_bss_skeleton_get_frequency (WPABSS *object)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (object);
  guint value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_uint (&(skeleton->priv->properties[3]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static guint 
wpa_bss_skeleton_get_age (WPABSS *object)
{
  WPABSSSkeleton *skeleton = WPA_BSS_SKELETON (object);
  guint value;
  g_mutex_lock (&skeleton->priv->lock);
  value = g_marshal_value_peek_uint (&(skeleton->priv->properties[4]));
  g_mutex_unlock (&skeleton->priv->lock);
  return value;
}

static void
wpa_bss_skeleton_class_init (WPABSSSkeletonClass *klass)
{
  GObjectClass *gobject_class;
  GDBusInterfaceSkeletonClass *skeleton_class;

  gobject_class = G_OBJECT_CLASS (klass);
  gobject_class->finalize = wpa_bss_skeleton_finalize;
  gobject_class->get_property = wpa_bss_skeleton_get_property;
  gobject_class->set_property = wpa_bss_skeleton_set_property;
  gobject_class->notify       = wpa_bss_skeleton_notify;


  wpa_bss_override_properties (gobject_class, 1);

  skeleton_class = G_DBUS_INTERFACE_SKELETON_CLASS (klass);
  skeleton_class->get_info = wpa_bss_skeleton_dbus_interface_get_info;
  skeleton_class->get_properties = wpa_bss_skeleton_dbus_interface_get_properties;
  skeleton_class->flush = wpa_bss_skeleton_dbus_interface_flush;
  skeleton_class->get_vtable = wpa_bss_skeleton_dbus_interface_get_vtable;

#if GLIB_VERSION_MAX_ALLOWED < GLIB_VERSION_2_38
  g_type_class_add_private (klass, sizeof (WPABSSSkeletonPrivate));
#endif
}

static void
wpa_bss_skeleton_iface_init (WPABSSIface *iface)
{
  iface->get_ssid = wpa_bss_skeleton_get_ssid;
  iface->get_bssid = wpa_bss_skeleton_get_bssid;
  iface->get_signal = wpa_bss_skeleton_get_signal;
  iface->get_frequency = wpa_bss_skeleton_get_frequency;
  iface->get_age = wpa_bss_skeleton_get_age;
}

/**
 * wpa_bss_skeleton_new:
 *
 * Creates a skeleton object for the D-Bus interface <link linkend="gdbus-interface-fi-w1-wpa_supplicant1-BSS.top_of_page">fi.w1.wpa_supplicant1.BSS</link>.
 *
 * Returns: (transfer full) (type WPABSSSkeleton): The skeleton object.
 */
WPABSS *
wpa_bss_skeleton_new (void)
{
  return WPA_BSS (g_object_new (TYPE_WPA_BSS_SKELETON, NULL));
}

